# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.obj as obj
import volatility.plugins.mac.common as common
import volatility.plugins.mac.list_files as mac_list_files

class mac_devfs(common.AbstractMacCommand):
    """ Lists files in the file cache """

    def calculate(self):
        common.set_plugin_members(self)

        nchrdev_addr = self.addr_space.profile.get_symbol("_nchrdev")
        nchrdev = obj.Object("unsigned int", offset = nchrdev_addr, vm = self.addr_space)

        cdevsw_addr = self.addr_space.profile.get_symbol("_cdevsw")
        cdevsw = obj.Object(theType = "Array", targetType = "cdevsw", offset = cdevsw_addr, vm = self.addr_space, count = nchrdev)

        kaddr_info = common.get_handler_name_addrs(self)

        op_members = self.profile.types['cdevsw'].keywords["members"].keys()
        
        op_members.remove('d_ttys')
        op_members.remove('d_type')

        files = mac_list_files.mac_list_files(self._config).calculate()
        for vnode, path in files:
            if vnode.v_type.v() not in [3, 4]:
                continue

            if path.startswith("/Macintosh HD"):
                path = path[13:]

            dn = vnode.v_data.dereference_as("devnode") 
 
            dev   = dn.dn_typeinfo.dev
            major = (dev >> 24) & 0xff

            if not (0 <= major <= nchrdev):
                continue
        
            cdev = cdevsw[major]
           
            for member in op_members:
                ptr = cdev.__getattr__(member).v()
        
                if ptr != 0: 
                    (module, handler_sym) = common.get_handler_name(kaddr_info, ptr)

                    yield (cdev.v(), path, member, ptr, module, handler_sym)

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Offset (V)", "[addrpad]"), 
                                  ("Path", "16"),
                                  ("Member", "16"),
                                  ("Handler", "[addrpad]"),
                                  ("Module",  "32"),
                                  ("Handler", "")])

        for (cdev, path, member, handler, module, sym) in data:
            self.table_row(outfd, cdev, path, member, handler, module, sym)


 



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""
import volatility.obj   as obj
import volatility.utils as utils
import volatility.debug as debug

import volatility.plugins.mac.common as common

class mac_kernel_classes(common.AbstractMacCommand):
    """ Lists loaded c++ classes in the kernel """

    def _struct_or_class(self, type_name):
        """Return the name of a structure or class. 

        More recent versions of OSX define some types as 
        classes instead of structures, so the naming is
        a little different.   
        """
        if self.addr_space.profile.vtypes.has_key(type_name):
            return type_name
        else:
            return type_name + "_class"
   
    def calculate(self):
        common.set_plugin_members(self)
        
        kaddr_info = common.get_handler_name_addrs(self)

        dict_ptr_addr = common.get_cpp_sym("sAllClassesDict", self.addr_space.profile)     
        dict_addr = obj.Object("unsigned long", offset = dict_ptr_addr, vm = self.addr_space)

        fdict = obj.Object(self._struct_or_class("OSDictionary"), offset = dict_addr.v(), vm = self.addr_space)
        
        ents = obj.Object('Array', offset = fdict.dictionary, 
                          vm = self.addr_space, 
                          targetType = self._struct_or_class("dictEntry"), 
                          count = fdict.count)

        for ent in ents:
            if ent == None or not ent.is_valid():
                continue
            
            class_name = str(ent.key.dereference_as(self._struct_or_class("OSString")))
           
            osmeta = obj.Object(self._struct_or_class("OSMetaClass"), offset = ent.value.v(), vm = self.addr_space)

            cname = str(osmeta.className.dereference_as(self._struct_or_class("OSString")))
            
            offset = 0

            if hasattr(osmeta, "metaClass"):
                arr_start = osmeta.metaClass.v()
            else:
                arr_start = obj.Object("Pointer", offset = osmeta.obj_offset, vm = self.addr_space)

            vptr = obj.Object("unsigned long", offset = arr_start, vm = self.addr_space)
            while vptr != 0:
                (module, handler_sym) = common.get_handler_name(kaddr_info, vptr)

                yield (cname, vptr, module, handler_sym)
                
                offset = offset + vptr.size()

                vptr = obj.Object("unsigned long", offset = arr_start + offset, vm = self.addr_space)
    
    def render_text(self, outfd, data):
        self.table_header(outfd, [("Class", "48"),
                                  ("Address", "[addrpad]"),
                                  ("Module", "48"),
                                  ("Handler", "")])

        for (cname, vptr, module, handler_sym) in data:
            self.table_row(outfd, cname, vptr, module, handler_sym) 















# Volatility
# Copyright (c) 2008-2015 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#
import volatility.utils as utils
import volatility.obj as obj
import volatility.debug as debug
import volatility.plugins.common as common
import volatility.plugins.malware.devicetree as dtree
import volatility.win32.modules as modules
import volatility.win32.tasks as tasks
from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class drivermodule(common.AbstractWindowsCommand):
    """Associate driver objects to kernel modules"""

    def __init__(self, config, *args, **kwargs):
        common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)
        config.add_option('ADDR', short_option = 'a', default = None,
                          help = 'Show info on module at or containing this (base) address',
                          action = 'store', type = 'int')
 
    def calculate(self):
        addr_space = utils.load_as(self._config)

        modlist = list(modules.lsmod(addr_space))
        mods = dict((addr_space.address_mask(mod.DllBase), mod) for mod in modlist)
        mod_addrs = sorted(mods.keys())
            
        drivers = dtree.DriverIrp(self._config).calculate()    
        driver_name = "UNKNOWN"
        service_key = "UNKNOWN"
        driver_name3 = "UNKNOWN"
        module_name = "UNKNOWN"

        if self._config.ADDR:
            find_address = self._config.ADDR
            
            module_name = tasks.find_module(mods, mod_addrs, mods.values()[0].obj_vm.address_mask(find_address))
            if module_name:
                module_name = module_name.BaseDllName or module_name.FullDllName

            for driver in drivers:
                if driver.DriverStart <= find_address < driver.DriverStart + driver.DriverSize:
                    header = driver.get_object_header()
                    driver_name = header.NameInfo.Name
                    driver_name  = str(driver.get_object_header().NameInfo.Name or '') 
                    service_key = str(driver.DriverExtension.ServiceKeyName or '') 
                    driver_name3 = str(driver.DriverName or '') 
                    break
            
            yield (module_name, driver_name, service_key, driver_name3)

        else:                
            for driver in drivers:
                driver_name  = str(driver.get_object_header().NameInfo.Name or '')
                service_key = str(driver.DriverExtension.ServiceKeyName or '')
                driver_name3 = str(driver.DriverName or '')
                
                owning_module = tasks.find_module(mods, mod_addrs, mods.values()[0].obj_vm.address_mask(driver.DriverStart))
                module_name = "UNKNOWN"
                if owning_module:
                    module_name = owning_module.BaseDllName or owning_module.FullDllName

                yield (module_name, driver_name, service_key, driver_name3)

    def generator(self, data):

        for module_name, driver_name, service_key, driver_name3 in data:
            yield( 0, [str(module_name), str(driver_name), str(service_key), str(driver_name3)])

    def unified_output(self, data):
        return TreeGrid([("Module", str),
                         ("Driver", str),
                         ("Alt. Name", str),
                         ("Service Key", str)],
                        self.generator(data))

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Module", "36"), ("Driver", "24"), ("Alt. Name", "24"), ("Service Key", "")])

        for module_name, driver_name, service_key, driver_name3 in data:
            self.table_row(outfd, module_name, driver_name, service_key, driver_name3)
       






# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License Version 2 as
# published by the Free Software Foundation.  You may not use, modify or
# distribute this program under any other version of the GNU General
# Public License.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""

import volatility.plugins.linux.pslist as linux_pslist
from volatility.renderers import TreeGrid

class linux_psenv(linux_pslist.linux_pslist):
    '''Gathers processes along with their static environment variables'''
    def unified_output(self, data):
        return TreeGrid([("Name", str),
                       ("Pid", int),
                       ("Environment", str)],
                        self.generator(data))

    def generator(self, data):
        for task in data:
            yield (0, [str(task.comm), int(task.pid), str(task.get_environment())])

    def render_text(self, outfd, data):
        outfd.write("{0:6s} {1:6s} {2:12s}\n".format("Name", "Pid", "Environment"))
        for task in data:
            outfd.write("{0:17s} {1:6s} {2:s}\n".format(str(task.comm), str(task.pid), task.get_environment()))



syscalls = [
    [
        "NtAccessCheck",
        "NtWorkerFactoryWorkerReady",
        "NtAcceptConnectPort",
        "NtYieldExecution",
        "NtWriteVirtualMemory",
        "NtWriteRequestData",
        "NtWriteFileGather",
        "NtWriteFile",
        "NtSetHighEventPair",
        "NtSetHighEventPair",
        "NtWaitForWorkViaWorkerFactory",
        "NtWaitForSingleObject",
        "NtWaitForMultipleObjects32",
        "NtWaitForMultipleObjects",
        "NtWaitForKeyedEvent",
        "NtWaitForDebugEvent",
        "NtWaitForAlertByThreadId",
        "NtVdmControl",
        "NtUnsubscribeWnfStateChange",
        "NtUpdateWnfStateData",
        "NtUnmapViewOfSection",
        "NtUnmapViewOfSectionEx",
        "NtUnlockVirtualMemory",
        "NtUnlockFile",
        "NtUnloadKeyEx",
        "NtUnloadKey2",
        "NtUnloadKey",
        "NtUnloadDriver",
        "NtUmsThreadYield",
        "NtTranslateFilePath",
        "NtTraceEvent",
        "NtTraceControl",
        "NtThawTransactions",
        "NtThawRegistry",
        "NtTestAlert",
        "NtTerminateThread",
        "NtTerminateProcess",
        "NtTerminateJobObject",
        "NtSystemDebugControl",
        "NtSuspendThread",
        "NtSuspendProcess",
        "NtSubscribeWnfStateChange",
        "NtStopProfile",
        "NtStartProfile",
        "NtSinglePhaseReject",
        "NtSignalAndWaitForSingleObject",
        "NtShutdownWorkerFactory",
        "NtShutdownSystem",
        "NtSetWnfProcessNotificationEvent",
        "NtSetVolumeInformationFile",
        "NtSetValueKey",
        "NtSetUuidSeed",
        "NtSetTimerResolution",
        "NtSetTimerEx",
        "NtSetTimer",
        "NtSetThreadExecutionState",
        "NtSetSystemTime",
        "NtSetSystemPowerState",
        "NtSetSystemInformation",
        "NtSetSystemEnvironmentValueEx",
        "NtSetSystemEnvironmentValue",
        "NtSetSecurityObject",
        "NtSetQuotaInformationFile",
        "NtSetHighEventPair",
        "NtSetHighEventPair",
        "NtSetLdtEntries",
        "NtSetIRTimer",
        "NtSetTimer2",
        "NtCancelTimer2",
        "NtSetIoCompletionEx",
        "NtSetIoCompletion",
        "NtSetIntervalProfile",
        "NtSetInformationWorkerFactory",
        "NtSetInformationTransactionManager",
        "NtSetInformationTransaction",
        "NtSetInformationToken",
        "NtSetInformationThread",
        "NtSetInformationResourceManager",
        "NtSetInformationProcess",
        "NtSetInformationObject",
        "NtSetInformationKey",
        "NtSetInformationJobObject",
        "NtSetInformationFile",
        "NtSetInformationEnlistment",
        "NtSetInformationDebugObject",
        "NtSetHighEventPair",
        "NtSetHighEventPair",
        "NtSetEventBoostPriority",
        "NtSetEvent",
        "NtSetEaFile",
        "NtSetDriverEntryOrder",
        "NtSetDefaultUILanguage",
        "NtSetDefaultLocale",
        "NtSetDefaultHardErrorPort",
        "NtSetDebugFilterState",
        "NtSetContextThread",
        "NtSetCachedSigningLevel2",
        "NtSetCachedSigningLevel",
        "NtSetBootOptions",
        "NtSetBootEntryOrder",
        "NtSerializeBoot",
        "NtSecureConnectPort",
        "NtSaveMergedKeys",
        "NtSaveKeyEx",
        "NtSaveKey",
        "NtRollforwardTransactionManager",
        "NtRollbackTransaction",
        "NtRollbackEnlistment",
        "NtRollbackComplete",
        "NtRevertContainerImpersonation",
        "NtResumeThread",
        "NtResumeProcess",
        "NtRestoreKey",
        "NtResetWriteWatch",
        "NtResetEvent",
        "NtRequestWaitReplyPort",
        "NtRequestPort",
        "NtReplyWaitReplyPort",
        "NtReplyWaitReceivePortEx",
        "NtReplyWaitReceivePort",
        "NtReplyPort",
        "NtReplacePartitionUnit",
        "NtReplaceKey",
        "NtRenameTransactionManager",
        "NtRenameKey",
        "NtRemoveProcessDebug",
        "NtRemoveIoCompletionEx",
        "NtRemoveIoCompletion",
        "NtReleaseWorkerFactoryWorker",
        "NtReleaseSemaphore",
        "NtReleaseMutant",
        "NtReleaseKeyedEvent",
        "NtRegisterThreadTerminatePort",
        "NtRegisterProtocolAddressInformation",
        "NtRecoverTransactionManager",
        "NtRecoverResourceManager",
        "NtRecoverEnlistment",
        "NtReadVirtualMemory",
        "NtReadRequestData",
        "NtReadOnlyEnlistment",
        "NtReadFileScatter",
        "NtReadFile",
        "NtRaiseHardError",
        "NtRaiseException",
        "NtQueueApcThreadEx",
        "NtQueueApcThread",
        "NtQueryAuxiliaryCounterFrequency",
        "NtQueryWnfStateData",
        "NtQueryWnfStateNameInformation",
        "NtQueryVolumeInformationFile",
        "NtQueryVirtualMemory",
        "NtQueryValueKey",
        "NtQueryTimerResolution",
        "NtQueryTimer",
        "NtQuerySystemTime",
        "NtQuerySystemInformationEx",
        "NtQuerySystemInformation",
        "NtQuerySystemEnvironmentValueEx",
        "NtQuerySystemEnvironmentValue",
        "NtQuerySymbolicLinkObject",
        "NtQuerySemaphore",
        "NtQuerySecurityPolicy",
        "NtQuerySecurityObject",
        "NtQuerySecurityAttributesToken",
        "NtQuerySection",
        "NtQueryQuotaInformationFile",
        "NtQueryPortInformationProcess",
        "NtQueryPerformanceCounter",
        "NtQueryOpenSubKeysEx",
        "NtQueryOpenSubKeys",
        "NtQueryObject",
        "NtQueryMutant",
        "NtQueryMultipleValueKey",
        "NtQueryLicenseValue",
        "NtQueryKey",
        "NtQueryIoCompletion",
        "NtQueryIntervalProfile",
        "NtQueryInstallUILanguage",
        "NtQueryInformationWorkerFactory",
        "NtQueryInformationTransactionManager",
        "NtQueryInformationTransaction",
        "NtQueryInformationToken",
        "NtQueryInformationThread",
        "NtQueryInformationResourceManager",
        "NtQueryInformationProcess",
        "NtQueryInformationPort",
        "NtQueryInformationJobObject",
        "NtQueryInformationFile",
        "NtQueryInformationEnlistment",
        "NtQueryInformationByName",
        "NtQueryInformationAtom",
        "NtQueryFullAttributesFile",
        "NtQueryEvent",
        "NtQueryEaFile",
        "NtQueryDriverEntryOrder",
        "NtQueryDirectoryObject",
        "NtQueryDirectoryFile",
        "NtQueryDefaultUILanguage",
        "NtQueryDefaultLocale",
        "NtQueryDebugFilterState",
        "NtQueryBootOptions",
        "NtQueryBootEntryOrder",
        "NtQueryAttributesFile",
        "NtPulseEvent",
        "NtProtectVirtualMemory",
        "NtPropagationFailed",
        "NtPropagationComplete",
        "NtPrivilegeObjectAuditAlarm",
        "NtPrivilegedServiceAuditAlarm",
        "NtPrivilegeCheck",
        "NtSetInformationVirtualMemory",
        "NtPrePrepareEnlistment",
        "NtPrePrepareComplete",
        "NtPrepareEnlistment",
        "NtPrepareComplete",
        "NtPowerInformation",
        "NtPlugPlayControl",
        "NtOpenTransactionManager",
        "NtOpenTransaction",
        "NtOpenTimer",
        "NtOpenThreadTokenEx",
        "NtOpenThreadToken",
        "NtOpenThread",
        "NtOpenSymbolicLinkObject",
        "NtOpenSession",
        "NtOpenSemaphore",
        "NtOpenSection",
        "NtOpenResourceManager",
        "NtOpenPartition",
        "NtOpenProcessTokenEx",
        "NtOpenProcessToken",
        "NtOpenProcess",
        "NtOpenPrivateNamespace",
        "NtOpenObjectAuditAlarm",
        "NtOpenMutant",
        "NtOpenKeyTransactedEx",
        "NtOpenKeyTransacted",
        "NtOpenKeyEx",
        "NtOpenKeyedEvent",
        "NtOpenKey",
        "NtOpenJobObject",
        "NtOpenIoCompletion",
        "NtOpenFile",
        "NtCreateEventPair",
        "NtOpenEvent",
        "NtOpenEnlistment",
        "NtOpenDirectoryObject",
        "NtNotifyChangeSession",
        "NtNotifyChangeMultipleKeys",
        "NtNotifyChangeKey",
        "NtNotifyChangeDirectoryFile",
        "NtManagePartition",
        "NtModifyDriverEntry",
        "NtModifyBootEntry",
        "NtMapViewOfSection",
        "NtMapUserPhysicalPagesScatter",
        "NtMapUserPhysicalPages",
        "NtMapCMFModule",
        "NtMakeTemporaryObject",
        "NtMakePermanentObject",
        "NtLockVirtualMemory",
        "NtLockRegistryKey",
        "NtLockProductActivationKeys",
        "NtLockFile",
        "NtLoadKeyEx",
        "NtLoadKey2",
        "NtLoadKey",
        "NtLoadHotPatch",
        "NtLoadEnclaveData",
        "NtLoadDriver",
        "NtListenPort",
        "NtIsUILanguageComitted",
        "NtIsSystemResumeAutomatic",
        "NtIsProcessInJob",
        "NtInitiatePowerAction",
        "NtInitializeRegistry",
        "NtInitializeNlsFiles",
        "NtInitializeEnclave",
        "NtImpersonateThread",
        "NtImpersonateClientOfPort",
        "NtImpersonateAnonymousToken",
        "NtGetWriteWatch",
        "NtGetNotificationResourceManager",
        "NtGetNlsSectionPtr",
        "NtGetNextThread",
        "NtGetNextProcess",
        "NtGetMUIRegistryInfo",
        "NtGetDevicePowerState",
        "NtGetCurrentProcessorNumberEx",
        "NtGetCurrentProcessorNumber",
        "NtGetContextThread",
        "NtGetCompleteWnfStateSubscription",
        "NtGetCachedSigningLevel",
        "NtFsControlFile",
        "NtFreezeTransactions",
        "NtFreezeRegistry",
        "NtFreeVirtualMemory",
        "NtFreeUserPhysicalPages",
        "NtFlushWriteBuffer",
        "NtFlushVirtualMemory",
        "NtFlushProcessWriteBuffers",
        "NtFlushKey",
        "FsRtlSyncVolumes",
        "NtFlushInstallUILanguage",
        "NtFlushBuffersFile",
        "NtFlushBuffersFileEx",
        "NtFindAtom",
        "NtFilterToken",
        "NtFilterTokenEx",
        "NtFilterBootOption",
        "NtExtendSection",
        "NtEnumerateValueKey",
        "NtEnumerateTransactionObject",
        "NtEnumerateSystemEnvironmentValuesEx",
        "NtEnumerateKey",
        "NtEnumerateDriverEntries",
        "NtEnumerateBootEntries",
        "NtEnableLastKnownGood",
        "NtDuplicateToken",
        "NtDuplicateObject",
        "NtDrawText",
        "NtDisplayString",
        "NtDisableLastKnownGood",
        "NtDeviceIoControlFile",
        "NtDeleteWnfStateName",
        "NtDeleteWnfStateData",
        "NtDeleteValueKey",
        "NtDeletePrivateNamespace",
        "NtDeleteObjectAuditAlarm",
        "NtDeleteKey",
        "NtDeleteFile",
        "NtDeleteDriverEntry",
        "NtDeleteBootEntry",
        "NtDeleteAtom",
        "NtDelayExecution",
        "NtDebugContinue",
        "NtDebugActiveProcess",
        "NtCreatePartition",
        "NtCreateWorkerFactory",
        "NtCreateWnfStateName",
        "NtCreateWaitCompletionPacket",
        "NtCreateWaitablePort",
        "NtCreateUserProcess",
        "NtCreateTransactionManager",
        "NtCreateTransaction",
        "NtCreateToken",
        "NtCreateLowBoxToken",
        "NtCreateTokenEx",
        "NtCreateTimer",
        "NtCreateThreadEx",
        "NtCreateThread",
        "NtCreateSymbolicLinkObject",
        "NtCreateSemaphore",
        "NtCreateSection",
        "NtCreateResourceManager",
        "NtCreateProfileEx",
        "NtCreateProfile",
        "NtCreateProcessEx",
        "NtCreateProcess",
        "NtCreatePrivateNamespace",
        "NtCreatePort",
        "NtCreatePagingFile",
        "NtCreateNamedPipeFile",
        "NtCreateMutant",
        "NtCreateMailslotFile",
        "NtCreateKeyTransacted",
        "NtCreateKeyedEvent",
        "NtCreateKey",
        "NtCreateJobSet",
        "NtCreateJobObject",
        "NtCreateIRTimer",
        "NtCreateTimer2",
        "NtCreateIoCompletion",
        "NtCreateFile",
        "NtCreateEventPair",
        "NtCreateEvent",
        "NtCreateEnlistment",
        "NtCreateEnclave",
        "NtCreateDirectoryObjectEx",
        "NtCreateDirectoryObject",
        "NtCreateDebugObject",
        "NtConvertBetweenAuxiliaryCounterAndPerformanceCounter",
        "NtContinue",
        "NtConnectPort",
        "NtCompressKey",
        "NtCompleteConnectPort",
        "NtCompareTokens",
        "NtCompareSigningLevels",
        "NtCompareObjects",
        "NtCompactKeys",
        "NtCommitTransaction",
        "NtCommitEnlistment",
        "NtCommitComplete",
        "NtCloseObjectAuditAlarm",
        "NtClose",
        "NtClearEvent",
        "NtCancelWaitCompletionPacket",
        "NtCancelTimer",
        "NtCancelSynchronousIoFile",
        "NtCancelIoFileEx",
        "NtCancelIoFile",
        "NtCallbackReturn",
        "NtAssociateWaitCompletionPacket",
        "NtAssignProcessToJobObject",
        "NtAreMappedFilesTheSame",
        "NtApphelpCacheControl",
        "NtAlpcSetInformation",
        "NtAlpcSendWaitReceivePort",
        "NtAlpcRevokeSecurityContext",
        "NtAlpcQueryInformationMessage",
        "NtAlpcQueryInformation",
        "NtAlpcOpenSenderThread",
        "NtAlpcOpenSenderProcess",
        "NtAlpcImpersonateClientOfPort",
        "NtAlpcImpersonateClientContainerOfPort",
        "NtAlpcDisconnectPort",
        "NtAlpcDeleteSecurityContext",
        "NtAlpcDeleteSectionView",
        "NtAlpcDeleteResourceReserve",
        "NtAlpcDeletePortSection",
        "NtAlpcCreateSecurityContext",
        "NtAlpcCreateSectionView",
        "NtAlpcCreateResourceReserve",
        "NtAlpcCreatePortSection",
        "NtAlpcCreatePort",
        "NtAlpcConnectPort",
        "NtAlpcConnectPortEx",
        "NtAlpcCancelMessage",
        "NtAlpcAcceptConnectPort",
        "NtAllocateVirtualMemory",
        "NtAllocateUuids",
        "NtAllocateUserPhysicalPages",
        "NtAllocateReserveObject",
        "NtAllocateLocallyUniqueId",
        "NtAlertThreadByThreadId",
        "NtAlertThread",
        "NtAlertResumeThread",
        "NtAdjustPrivilegesToken",
        "NtAdjustGroupsToken",
        "NtAdjustTokenClaimsAndDeviceGroups",
        "NtAddDriverEntry",
        "NtAddBootEntry",
        "NtAddAtom",
        "NtAddAtomEx",
        "NtAcquireProcessActivityReference",
        "NtAccessCheckByTypeResultListAndAuditAlarmByHandle",
        "NtAccessCheckByTypeResultListAndAuditAlarm",
        "NtAccessCheckByTypeResultList",
        "NtAccessCheckByTypeAndAuditAlarm",
        "NtAccessCheckByType",
        "NtAccessCheckAndAuditAlarm",
        "NtSetInformationSymbolicLink",
        "NtCreateRegistryTransaction",
        "NtOpenRegistryTransaction",
        "NtCommitRegistryTransaction",
        "NtRollbackRegistryTransaction"
    ],
    [
        "NtUserGetOwnerTransformedMonitorRect",
        "NtUserYieldTask",
        "NtUserSetSensorPresence",
        "NtGdiWidenPath",
        "NtGdiUpdateColors",
        "NtGdiUnrealizeObject",
        "NtGdiUnmapMemFont",
        "NtGdiUnloadPrinterDriver",
        "NtGdiTransparentBlt",
        "NtGdiTransformPoints",
        "NtGdiScaleRgn",
        "NtGdiScaleValues",
        "NtGdiGetDCDpiScaleValue",
        "NtGdiGetBitmapDpiScaleValue",
        "NtGdiSwapBuffers",
        "NtGdiStrokePath",
        "NtGdiStrokeAndFillPath",
        "NtGdiStretchDIBitsInternal",
        "NtGdiStretchBlt",
        "NtGdiStartPage",
        "NtGdiStartDoc",
        "NtGdiSetSizeDevice",
        "NtGdiSetVirtualResolution",
        "NtGdiSetTextJustification",
        "NtGdiSetSystemPaletteUse",
        "NtGdiSetRectRgn",
        "NtGdiSetPixelFormat",
        "NtGdiSetPixel",
        "NtGdiSetOPMSigningKeyAndSequenceNumbers",
        "NtGdiSetLayout",
        "NtGdiMirrorWindowOrg",
        "NtGdiGetDeviceWidth",
        "NtGdiSetMiterLimit",
        "NtGdiSetMetaRgn",
        "NtGdiSetMagicColors",
        "NtGdiSetLinkedUFIs",
        "NtGdiSetIcmMode",
        "NtGdiSetFontXform",
        "NtGdiSetFontEnumeration",
        "NtGdiSetDIBitsToDeviceInternal",
        "NtGdiSetDeviceGammaRamp",
        "NtGdiGetGammaRampCapability",
        "NtGdiSetPrivateDeviceGammaRamp",
        "NtGdiGetAppliedDeviceGammaRamp",
        "NtGdiSetColorSpace",
        "NtGdiSetColorAdjustment",
        "NtGdiSetBrushOrg",
        "NtGdiSetBrushAttributes",
        "NtGdiSetBoundsRect",
        "NtGdiSetBitmapDimension",
        "NtGdiSetBitmapBits",
        "NtGdiSetBitmapAttributes",
        "NtGdiSelectPen",
        "NtGdiSelectFont",
        "NtGdiSelectClipPath",
        "NtGdiSelectBrush",
        "NtGdiSelectBitmap",
        "NtGdiScaleWindowExtEx",
        "NtGdiScaleViewportExtEx",
        "NtGdiSaveDC",
        "NtGdiRoundRect",
        "NtGdiRestoreDC",
        "NtGdiResizePalette",
        "NtGdiResetDC",
        "NtGdiRemoveFontMemResourceEx",
        "NtGdiRemoveFontResourceW",
        "NtGdiRectVisible",
        "NtGdiRectInRegion",
        "NtGdiRectangle",
        "NtGdiQueryFontAssocInfo",
        "NtGdiQueryFonts",
        "NtGdiPtVisible",
        "NtGdiPtInRegion",
        "NtGdiPolyTextOutW",
        "NtGdiPolyPolyDraw",
        "NtGdiPolyDraw",
        "NtGdiPlgBlt",
        "NtGdiPathToRegion",
        "NtGdiPolyPatBlt",
        "NtGdiPatBlt",
        "NtGdiOpenDCW",
        "NtGdiOffsetRgn",
        "NtGdiOffsetClipRgn",
        "NtGdiMoveTo",
        "NtGdiMonoBitmap",
        "NtGdiModifyWorldTransform",
        "NtGdiMaskBlt",
        "NtGdiMakeInfoDC",
        "NtGdiMakeFontDir",
        "NtGdiLineTo",
        "NtGdiInvertRgn",
        "NtGdiIntersectClipRect",
        "NtGdiInitSpool",
        "NtGdiInit",
        "NtGdiIcmBrushInfo",
        "NtGdiHfontCreate",
        "NtGdiGradientFill",
        "NtGdiGetWidthTable",
        "NtGdiGetFontUnicodeRanges",
        "NtGdiAddEmbFontToDC",
        "NtGdiChangeGhostFont",
        "NtGdiGetEmbedFonts",
        "NtGdiGetUFIPathname",
        "NtGdiGetEmbUFI",
        "NtGdiGetUFI",
        "NtGdiGetTransform",
        "NtGdiGetTextMetricsW",
        "NtGdiGetTextFaceW",
        "NtGdiGetTextExtentExW",
        "NtGdiGetTextExtent",
        "NtGdiGetTextCharsetInfo",
        "NtGdiGetSystemPaletteUse",
        "NtGdiGetSuggestedOPMProtectedOutputArraySize",
        "NtGdiGetStringBitmapW",
        "NtGdiGetStats",
        "NtGdiGetSpoolMessage",
        "NtGdiGetServerMetaFileBits",
        "NtGdiGetRgnBox",
        "NtGdiGetRegionData",
        "NtGdiGetRealizationInfo",
        "NtGdiGetRasterizerCaps",
        "NtGdiGetRandomRgn",
        "NtGdiGetPixel",
        "NtGdiGetPath",
        "NtGdiGetOutlineTextMetricsInternalW",
        "NtGdiGetOPMRandomNumber",
        "NtGdiGetObjectBitmapHandle",
        "NtGdiGetNearestPaletteIndex",
        "NtGdiGetNearestColor",
        "NtGdiGetMonitorID",
        "NtGdiGetMiterLimit",
        "NtGdiGetLinkedUFIs",
        "NtGdiGetKerningPairs",
        "NtGdiGetOPMInformation",
        "NtGdiGetGlyphOutline",
        "NtGdiGetGlyphIndicesWInternal",
        "NtGdiGetGlyphIndicesW",
        "NtGdiGetFontResourceInfoInternalW",
        "NtGdiGetFontFileInfo",
        "NtGdiGetFontFileData",
        "NtGdiGetFontData",
        "NtGdiGetEudcTimeStampEx",
        "NtGdiGetETM",
        "NtGdiGetEntry",
        "NtGdiGetDIBitsInternal",
        "NtGdiGetDeviceCapsAll",
        "NtGdiGetDeviceGammaRamp",
        "NtGdiGetDeviceCaps",
        "NtGdiGetDCPoint",
        "NtGdiGetDCObject",
        "NtGdiGetDCforBitmap",
        "NtGdiGetDCDword",
        "NtGdiGetCurrentDpiInfo",
        "NtGdiGetCOPPCompatibleOPMInformation",
        "NtGdiGetColorSpaceforBitmap",
        "NtGdiGetColorAdjustment",
        "NtGdiGetCharWidthInfo",
        "NtGdiGetCharWidthW",
        "NtGdiGetCharSet",
        "NtGdiGetCharacterPlacementW",
        "NtGdiGetCharABCWidthsW",
        "NtGdiGetCertificateSize",
        "NtGdiGetCertificate",
        "NtGdiGetCertificateSizeByHandle",
        "NtGdiGetCertificateByHandle",
        "NtGdiGetBoundsRect",
        "NtGdiGetBitmapDimension",
        "NtGdiGetBitmapBits",
        "NtGdiGetAppClipBox",
        "NtGdiGetAndSetDCDword",
        "NtGdiFullscreenControl",
        "NtGdiFrameRgn",
        "NtGdiForceUFIMapping",
        "NtGdiFlush",
        "NtGdiFlattenPath",
        "NtGdiFillRgn",
        "NtGdiFillPath",
        "NtGdiExtTextOutW",
        "NtGdiExtSelectClipRgn",
        "NtGdiExtGetObjectW",
        "NtGdiExtFloodFill",
        "NtGdiExtEscape",
        "NtGdiExtCreateRegion",
        "NtGdiExtCreatePen",
        "NtGdiExcludeClipRect",
        "NtGdiEudcLoadUnloadLink",
        "NtGdiEqualRgn",
        "NtGdiEnumObjects",
        "NtGdiEnumFonts",
        "NtGdiEndPath",
        "NtGdiEndPage",
        "NtGdiEndGdiRendering",
        "NtGdiEndDoc",
        "NtGdiEnableEudc",
        "NtGdiEllipse",
        "NtGdiDrawEscape",
        "NtGdiDoPalette",
        "NtGdiDoBanding",
        "NtGdiGetPerBandInfo",
        "NtGdiDestroyOPMProtectedOutput",
        "NtGdiDescribePixelFormat",
        "NtGdiDeleteObjectApp",
        "NtGdiDeleteColorTransform",
        "NtGdiDeleteColorSpace",
        "NtGdiDeleteClientObj",
        "NtGdiCreateSolidBrush",
        "NtGdiCreateServerMetaFile",
        "NtGdiCreateRoundRectRgn",
        "NtGdiCreateRectRgn",
        "NtGdiCreatePen",
        "NtGdiCreatePatternBrushInternal",
        "NtGdiCreatePaletteInternal",
        "NtGdiCreateOPMProtectedOutputs",
        "NtGdiCreateOPMProtectedOutput",
        "NtGdiCreateMetafileDC",
        "NtGdiCreateHatchBrushInternal",
        "NtGdiCreateHalftonePalette",
        "NtGdiCreateEllipticRgn",
        "NtGdiCreateSessionMappedDIBSection",
        "NtGdiCreateDIBSection",
        "NtGdiCreateDIBitmapInternal",
        "NtGdiCreateDIBBrush",
        "NtGdiCreateCompatibleDC",
        "NtGdiCreateCompatibleBitmap",
        "NtGdiCreateColorTransform",
        "NtGdiCreateColorSpace",
        "NtGdiCreateClientObj",
        "NtGdiCreateBitmapFromDxSurface2",
        "NtGdiCreateBitmapFromDxSurface",
        "NtGdiCreateBitmap",
        "NtGdiConvertMetafileRect",
        "NtGdiConfigureOPMProtectedOutput",
        "NtGdiComputeXformCoefficients",
        "NtGdiCombineTransform",
        "NtGdiCombineRgn",
        "NtGdiColorCorrectPalette",
        "NtGdiClearBrushAttributes",
        "NtGdiClearBitmapAttributes",
        "NtGdiCloseFigure",
        "NtGdiCheckBitmapBits",
        "NtGdiCancelDC",
        "NtGdiBitBlt",
        "NtGdiBeginPath",
        "NtGdiBeginGdiRendering",
        "NtGdiArcInternal",
        "NtGdiFontIsLinked",
        "NtGdiAnyLinkedFonts",
        "NtGdiAngleArc",
        "NtGdiAlphaBlend",
        "NtGdiAddRemoteMMInstanceToDC",
        "NtGdiRemoveMergeFont",
        "NtGdiAddFontMemResourceEx",
        "NtGdiAddRemoteFontToDC",
        "NtGdiAddFontResourceW",
        "NtGdiAbortPath",
        "NtGdiAbortDoc",
        "NtUserDefSetText",
        "NtUserDeferWindowPosAndBand",
        "NtUserDdeInitialize",
        "NtUserCanBrokerForceForeground",
        "NtUserCreateWindowStation",
        "NtUserCreateWindowEx",
        "NtUserCreateLocalMemHandle",
        "NtUserCreateInputContext",
        "NtUserCreateDesktopEx",
        "NtUserCreateCaret",
        "NtUserCreateAcceleratorTable",
        "NtUserCountClipboardFormats",
        "NtUserCopyAcceleratorTable",
        "NtUserConvertMemHandle",
        "NtUserConsoleControl",
        "NtUserCloseWindowStation",
        "NtUserCloseDesktop",
        "NtUserCloseClipboard",
        "NtUserClipCursor",
        "NtUserChildWindowFromPointEx",
        "NtUserCheckMenuItem",
        "NtUserCheckWindowThreadDesktop",
        "NtUserDwmValidateWindow",
        "NtUserCheckAccessForIntegrityLevel",
        "NtUserDisplayConfigSetDeviceInfo",
        "NtUserDisplayConfigGetDeviceInfo",
        "NtUserQueryDisplayConfig",
        "NtUserSetDisplayConfig",
        "NtUserFunctionalizeDisplayConfig",
        "NtUserGetDisplayConfigBufferSizes",
        "NtUserChangeDisplaySettings",
        "NtUserChangeClipboardChain",
        "NtUserCallTwoParam",
        "NtUserCallOneParam",
        "NtUserCallNoParam",
        "NtUserCallNextHookEx",
        "NtUserCallMsgFilter",
        "NtUserCallHwndParamLock",
        "NtUserCallHwndParam",
        "NtUserCallHwndOpt",
        "NtUserCallHwndLock",
        "NtUserCallHwnd",
        "NtUserBroadcastThemeChangeEvent",
        "NtUserBuildPropList",
        "NtUserBuildNameList",
        "NtUserBuildHwndList",
        "NtUserBuildHimcList",
        "NtUserBlockInput",
        "NtUserBitBltSysBmp",
        "NtUserBeginPaint",
        "NtUserAttachThreadInput",
        "NtUserAssociateInputContext",
        "NtUserAlterWindowStyle",
        "NtUserAddClipboardFormatListener",
        "NtUserActivateKeyboardLayout",
        "NtUserDelegateCapturePointers",
        "NtUserDelegateInput",
        "NtUserDispatchMessage",
        "NtUserDisableProcessWindowFiltering",
        "NtUserDisableThreadIme",
        "NtUserDestroyWindow",
        "NtUserDestroyMenu",
        "NtUserDestroyInputContext",
        "NtUserDestroyCursor",
        "NtUserDestroyAcceleratorTable",
        "NtUserDeleteMenu",
        "NtUserDoSoundDisconnect",
        "NtUserDoSoundConnect",
        "NtUserGhostWindowFromHungWindow",
        "NtUserGetWOWClass",
        "NtUserGetWindowPlacement",
        "NtUserGetWindowDisplayAffinity",
        "NtUserGetWindowDC",
        "NtUserGetWindowCompositionAttribute",
        "NtUserGetWindowCompositionInfo",
        "NtUserGetWindowBand",
        "NtUserGetUpdateRgn",
        "NtUserGetUpdateRect",
        "NtUserGetUpdatedClipboardFormats",
        "NtUserGetTopLevelWindow",
        "NtUserGetTitleBarInfo",
        "NtUserGetThreadState",
        "NtUserGetThreadDesktop",
        "NtUserGetSystemMenu",
        "NtUserGetScrollBarInfo",
        "NtUserGetRegisteredRawInputDevices",
        "NtUserGetRawInputDeviceList",
        "NtUserGetRawInputDeviceInfo",
        "NtUserGetRawInputData",
        "NtUserGetRawInputBuffer",
        "NtUserGetProcessDpiAwarenessContext",
        "NtUserGetProcessWindowStation",
        "NtUserGetPriorityClipboardFormat",
        "NtUserGetOpenClipboardWindow",
        "NtUserGetObjectInformation",
        "NtUserGetMouseMovePointsEx",
        "NtUserGetMessage",
        "NtUserGetMenuItemRect",
        "NtUserGetMenuIndex",
        "NtUserGetMenuBarInfo",
        "NtUserGetListBoxInfo",
        "NtUserGetKeyState",
        "NtUserGetKeyNameText",
        "NtUserGetKeyboardState",
        "NtUserGetKeyboardLayoutName",
        "NtUserGetKeyboardLayoutList",
        "NtUserGetInternalWindowPos",
        "NtUserGetInputLocaleInfo",
        "NtUserGetImeInfoEx",
        "NtUserGetImeHotKey",
        "NtUserGetIconSize",
        "NtUserGetIconInfo",
        "NtUserGetGUIThreadInfo",
        "NtUserGetGuiResources",
        "NtUserGetForegroundWindow",
        "NtUserGetDoubleClickTime",
        "NtUserGetDesktopID",
        "NtUserGetDCEx",
        "NtUserGetDC",
        "NtUserGetCursorInfo",
        "NtUserGetCursorFrameInfo",
        "NtUserGetCurrentInputMessageSource",
        "NtUserGetCIMSSM",
        "NtUserGetCPD",
        "NtUserGetControlColor",
        "NtUserGetControlBrush",
        "NtUserGetComboBoxInfo",
        "NtUserGetClipCursor",
        "NtUserGetClipboardViewer",
        "NtUserGetClipboardSequenceNumber",
        "NtUserGetClipboardOwner",
        "NtUserGetClipboardFormatName",
        "NtUserGetClipboardData",
        "NtUserGetClassName",
        "NtUserGetClassInfoEx",
        "NtUserGetCaretPos",
        "NtUserGetCaretBlinkTime",
        "NtUserGetAtomName",
        "NtUserGetAsyncKeyState",
        "NtUserGetAppImeLevel",
        "NtUserGetAncestor",
        "NtUserGetAltTabInfo",
        "NtUserFrostCrashedWindow",
        "NtUserFlashWindowEx",
        "NtUserFindWindowEx",
        "NtUserFindExistingCursorIcon",
        "NtUserFillWindow",
        "NtUserExcludeUpdateRgn",
        "NtUserEvent",
        "NtUserEnumDisplaySettings",
        "NtUserEnumDisplayMonitors",
        "NtUserEnumDisplayDevices",
        "NtUserEndPaint",
        "NtUserEndMenu",
        "NtUserEndDeferWindowPosEx",
        "NtUserEnableScrollBar",
        "NtUserEnableMenuItem",
        "NtUserEmptyClipboard",
        "NtUserDrawMenuBarTemp",
        "NtUserDrawIconEx",
        "NtUserDrawCaptionTemp",
        "NtUserDrawCaption",
        "NtUserDrawAnimatedRects",
        "NtUserDragObject",
        "NtUserDragDetect",
        "NtUserHandleDelegatedInput",
        "NtUserInheritWindowMonitor",
        "NtUserRealChildWindowFromPoint",
        "NtUserQueryWindow",
        "NtUserQuerySendMessage",
        "NtUserQueryInputContext",
        "NtUserQueryInformationThread",
        "NtUserQueryBSDRWindow",
        "NtUserPerMonitorDPIPhysicalToLogicalPoint",
        "NtUserProcessConnect",
        "NtUserPrintWindow",
        "NtUserPostThreadMessage",
        "NtUserPostMessage",
        "NtUserPhysicalToLogicalPoint",
        "NtUserPeekMessage",
        "NtUserPaintMonitor",
        "NtUserPaintDesktop",
        "NtUserOpenWindowStation",
        "NtUserOpenThreadDesktop",
        "NtUserOpenInputDesktop",
        "NtUserOpenDesktop",
        "NtUserOpenClipboard",
        "NtUserNotifyWinEvent",
        "NtUserNotifyProcessCreate",
        "NtUserNotifyIMEStatus",
        "NtUserMoveWindow",
        "NtUserModifyUserStartupInfoFlags",
        "NtUserMNDragOver",
        "NtUserMNDragLeave",
        "NtUserMinMaximize",
        "NtUserMessageCall",
        "NtUserMenuItemFromPoint",
        "NtUserMapVirtualKeyEx",
        "NtUserLayoutCompleted",
        "NtUserLogicalToPerMonitorDPIPhysicalPoint",
        "NtUserLogicalToPhysicalPoint",
        "NtUserLockCursor",
        "NtUserLockWorkStation",
        "NtUserLockWindowUpdate",
        "NtUserLockWindowStation",
        "NtUserLoadKeyboardLayoutEx",
        "NtUserKillTimer",
        "NtUserIsTopLevelWindow",
        "NtUserIsClipboardFormatAvailable",
        "NtUserInvalidateRgn",
        "NtUserInvalidateRect",
        "NtUserInternalGetWindowIcon",
        "NtUserInternalGetWindowText",
        "NtUserInitTask",
        "NtUserInitializeClientPfnArrays",
        "NtUserInitialize",
        "NtUserImpersonateDdeClientWindow",
        "NtUserHungWindowFromGhostWindow",
        "NtUserHiliteMenuItem",
        "NtUserHideCaret",
        "NtUserHardErrorControl",
        "NtUserRealInternalGetMessage",
        "NtUserRealWaitMessageEx",
        "NtUserReleaseDwmHitTestWaiters",
        "NtUserTranslateMessage",
        "NtUserTranslateAccelerator",
        "NtUserPaintMenuBar",
        "NtUserCalcMenuBar",
        "NtUserCalculatePopupWindowPosition",
        "NtUserTrackPopupMenuEx",
        "NtUserTrackMouseEvent",
        "NtUserToUnicodeEx",
        "NtUserThunkedMenuItemInfo",
        "NtUserThunkedMenuInfo",
        "NtUserTestForInteractiveUser",
        "NtUserSendEventMessage",
        "NtUserSystemParametersInfo",
        "NtUserSystemParametersInfoForDpi",
        "NtUserSwitchDesktop",
        "NtUserSoundSentry",
        "NtUserShutdownReasonDestroy",
        "NtUserShutdownBlockReasonQuery",
        "NtUserShutdownBlockReasonCreate",
        "NtUserShowWindowAsync",
        "NtUserShowWindow",
        "NtUserShowScrollBar",
        "NtUserShowCaret",
        "NtUserSetWinEventHook",
        "NtUserSetWindowWord",
        "NtUserSetWindowStationUser",
        "NtUserSetWindowsHookEx",
        "NtUserSetWindowsHookAW",
        "NtUserSetWindowRgnEx",
        "NtUserGetWindowRgnEx",
        "NtUserSetWindowRgn",
        "NtUserSetWindowPos",
        "NtUserSetWindowPlacement",
        "NtUserSetWindowLong",
        "NtUserSetWindowFNID",
        "NtUserSetWindowDisplayAffinity",
        "NtUserSetWindowCompositionTransition",
        "NtUserUpdateDefaultDesktopThumbnail",
        "NtUserSetWindowCompositionAttribute",
        "NtUserSetWindowBand",
        "NtUserSetProcessUIAccessZorder",
        "NtUserSetProcessDpiAwarenessContext",
        "NtUserEnableChildWindowDpiMessage",
        "NtUserIsChildWindowDpiMessageEnabled",
        "NtUserEnableNonClientDpiScaling",
        "NtUserIsNonClientDpiScalingEnabled",
        "NtUserSetDialogControlDpiChangeBehavior",
        "NtUserIsWindowBroadcastingDpiToChildren",
        "NtUserEnableWindowGDIScaledDpiMessage",
        "NtUserIsWindowGDIScaledDpiMessageEnabled",
        "NtUserSetTimer",
        "NtUserSetThreadState",
        "NtUserSetThreadLayoutHandles",
        "NtUserSetThreadDesktop",
        "NtUserSetThreadInputBlocked",
        "NtUserSetSystemTimer",
        "NtUserSetSystemMenu",
        "NtUserSetSystemCursor",
        "NtUserSetSysColors",
        "NtUserSetShellWindowEx",
        "NtUserSetScrollInfo",
        "NtUserSetProp",
        "NtUserGetProp",
        "NtUserSetProcessWindowStation",
        "NtUserSetParent",
        "NtUserSetObjectInformation",
        "NtUserSetMenuFlagRtoL",
        "NtUserSetMenuDefaultItem",
        "NtUserSetMenuContextHelpId",
        "NtUserSetMenu",
        "NtUserSetKeyboardState",
        "NtUserSetInternalWindowPos",
        "NtUserSetInformationThread",
        "NtUserSetImeOwnerWindow",
        "NtUserSetImeInfoEx",
        "NtUserSetImeHotKey",
        "NtUserSetFocus",
        "NtUserSetCursorIconData",
        "NtUserSetCursorContents",
        "NtUserSetCursor",
        "NtUserSetClipboardViewer",
        "NtUserSetClipboardData",
        "NtUserSetClassWord",
        "NtUserSetClassLong",
        "NtUserSetChildWindowNoActivate",
        "NtUserSetCapture",
        "NtUserSetAppImeLevel",
        "NtUserSetActiveWindow",
        "NtUserSendInput",
        "NtUserSelectPalette",
        "NtUserScrollWindowEx",
        "NtUserScrollDC",
        "NtUserSBGetParms",
        "NtUserResolveDesktopForWOW",
        "NtUserRemoveProp",
        "NtUserRemoveMenu",
        "NtUserRemoveClipboardFormatListener",
        "NtUserRegisterWindowMessage",
        "NtUserRegisterTasklist",
        "NtUserRegisterServicesProcess",
        "NtUserRegisterRawInputDevices",
        "NtUserRegisterHotKey",
        "NtUserRegisterDManipHook",
        "NtUserGetDManipHookInitFunction",
        "NtUserRegisterManipulationThread",
        "NtUserSetManipulationInputTarget",
        "NtUserRegisterUserApiHook",
        "NtUserRegisterErrorReportingDialog",
        "NtUserRegisterClassExWOW",
        "NtUserRegisterBSDRWindow",
        "NtUserRedrawWindow",
        "NtUserUndelegateInput",
        "NtUserGetWindowMinimizeRect",
        "NtUserDwmGetRemoteSessionOcclusionEvent",
        "NtUserDwmGetRemoteSessionOcclusionState",
        "NtUserDwmKernelShutdown",
        "NtUserDwmKernelStartup",
        "NtUserCheckProcessSession",
        "NtUserUnregisterSessionPort",
        "NtUserRegisterSessionPort",
        "NtUserCtxDisplayIOCtl",
        "NtUserRemoteStopScreenUpdates",
        "NtUserRemoteRedrawScreen",
        "NtUserRemoteRedrawRectangle",
        "NtUserRemoteConnect",
        "NtUserWaitAvailableMessageEx",
        "NtUserWindowFromPoint",
        "NtUserWindowFromPhysicalPoint",
        "NtUserWaitMessage",
        "NtUserWaitForMsgAndEvent",
        "NtUserWaitForInputIdle",
        "NtUserVkKeyScanEx",
        "NtUserValidateTimerCallback",
        "NtUserValidateRect",
        "NtUserValidateHandleSecure",
        "NtUserUserHandleGrantAccess",
        "NtUserUpdatePerUserSystemParameters",
        "NtUserSetLayeredWindowAttributes",
        "NtUserGetLayeredWindowAttributes",
        "NtUserUpdateLayeredWindow",
        "NtUserUpdateInstance",
        "NtUserUpdateInputContext",
        "NtUserUnregisterHotKey",
        "NtUserUnregisterUserApiHook",
        "NtUserUnregisterClass",
        "NtUserUnlockWindowStation",
        "NtUserUnloadKeyboardLayout",
        "NtUserUnhookWinEvent",
        "NtUserUnhookWindowsHookEx",
        "NtUserGetTouchInputInfo",
        "NtUserIsTouchWindow",
        "NtUserModifyWindowTouchCapability",
        "NtGdiEngStretchBltROP",
        "NtGdiEngTextOut",
        "NtGdiEngTransparentBlt",
        "NtGdiEngGradientFill",
        "NtGdiEngAlphaBlend",
        "NtGdiEngLineTo",
        "NtGdiEngPaint",
        "NtGdiEngStrokeAndFillPath",
        "NtGdiEngFillPath",
        "NtGdiEngStrokePath",
        "NtGdiEngMarkBandingSurface",
        "NtGdiEngPlgBlt",
        "NtGdiEngStretchBlt",
        "NtGdiEngBitBlt",
        "NtGdiEngLockSurface",
        "NtGdiEngUnlockSurface",
        "NtGdiEngEraseSurface",
        "NtGdiEngDeleteSurface",
        "NtGdiEngDeletePalette",
        "NtGdiEngCopyBits",
        "NtGdiEngComputeGlyphSet",
        "NtGdiEngCreatePalette",
        "NtGdiEngCreateDeviceBitmap",
        "NtGdiEngCreateDeviceSurface",
        "NtGdiEngCreateBitmap",
        "NtGdiEngAssociateSurface",
        "NtUserSetWindowFeedbackSetting",
        "NtUserRegisterEdgy",
        "NtUserRegisterShellPTPListener",
        "NtUserGetWindowFeedbackSetting",
        "NtUserHidePointerContactVisualization",
        "NtUserGetTouchValidationStatus",
        "NtUserInitializeTouchInjection",
        "NtUserInjectTouchInput",
        "NtUserRegisterTouchHitTestingWindow",
        "NtUserSetDisplayMapping",
        "NtUserSetCalibrationData",
        "NtUserGetPhysicalDeviceRect",
        "NtUserRegisterTouchPadCapable",
        "NtUserGetRawPointerDeviceData",
        "NtUserGetPointerDeviceCursors",
        "NtUserGetPointerDeviceRects",
        "NtUserRegisterPointerDeviceNotifications",
        "NtUserGetPointerDeviceProperties",
        "NtUserGetPointerDevice",
        "NtUserGetPointerDevices",
        "NtUserEnableTouchPad",
        "NtUserGetPrecisionTouchPadConfiguration",
        "NtUserSetPrecisionTouchPadConfiguration",
        "NtUserPromotePointer",
        "NtUserDiscardPointerFrameMessages",
        "NtUserRegisterPointerInputTarget",
        "NtUserGetPointerFrameArrivalTimes",
        "NtUserGetPointerInputTransform",
        "NtUserGetPointerInfoList",
        "NtUserGetPointerCursorId",
        "NtUserGetPointerType",
        "NtUserGetGestureConfig",
        "NtUserSetGestureConfig",
        "NtUserGetGestureExtArgs",
        "NtUserGetGestureInfo",
        "NtUserInjectGesture",
        "NtUserChangeWindowMessageFilterEx",
        "NtGdiXLATEOBJ_hGetColorTransform",
        "NtGdiXLATEOBJ_iXlate",
        "NtGdiXLATEOBJ_cGetPalette",
        "NtGdiEngDeleteClip",
        "NtGdiEngCreateClip",
        "NtGdiEngDeletePath",
        "NtGdiCLIPOBJ_ppoGetPath",
        "NtGdiCLIPOBJ_cEnumStart",
        "NtGdiCLIPOBJ_bEnum",
        "NtGdiBRUSHOBJ_hGetColorTransform",
        "NtGdiBRUSHOBJ_pvGetRbrush",
        "NtGdiBRUSHOBJ_pvAllocRbrush",
        "NtGdiBRUSHOBJ_ulGetBrushColor",
        "NtGdiXFORMOBJ_iGetXform",
        "NtGdiXFORMOBJ_bApplyXform",
        "NtGdiFONTOBJ_pQueryGlyphAttrs",
        "NtGdiFONTOBJ_pfdg",
        "NtGdiFONTOBJ_pifi",
        "NtGdiFONTOBJ_cGetGlyphs",
        "NtGdiFONTOBJ_pxoGetXform",
        "NtGdiFONTOBJ_vGetInfo",
        "NtGdiFONTOBJ_cGetAllGlyphHandles",
        "NtGdiFONTOBJ_pvTrueTypeFontFile",
        "NtGdiSTROBJ_dwGetCodePage",
        "NtGdiSTROBJ_vEnumStart",
        "NtGdiSTROBJ_bGetAdvanceWidths",
        "NtGdiSTROBJ_bEnumPositionsOnly",
        "NtGdiSTROBJ_bEnum",
        "NtGdiPATHOBJ_bEnumClipLines",
        "NtGdiPATHOBJ_vEnumStartClipLines",
        "NtGdiPATHOBJ_vEnumStart",
        "NtGdiPATHOBJ_bEnum",
        "NtGdiPATHOBJ_vGetBounds",
        "NtGdiEngCheckAbort",
        "NtGdiGetDhpdev",
        "NtGdiHT_Get8BPPMaskPalette",
        "NtGdiHT_Get8BPPFormatPalette",
        "NtGdiUpdateTransform",
        "NtGdiUMPDEngFreeUserMem",
        "NtGdiBRUSHOBJ_DeleteRbrush",
        "NtGdiSetPUMPDOBJ",
        "NtGdiSetUMPDSandboxState",
        "NtGdiDrawStream",
        "NtGdiHLSurfSetInformation",
        "NtGdiHLSurfGetInformation",
        "NtGdiDwmCreatedBitmapRemotingOutput",
        "NtGdiDdDDIGetScanLine",
        "NtGdiDdDDIReleaseProcessVidPnSourceOwners",
        "NtGdiDdDDIGetProcessSchedulingPriorityClass",
        "NtGdiDdDDISetProcessSchedulingPriorityClass",
        "NtGdiDdDDIGetContextSchedulingPriority",
        "NtGdiDdDDISetContextSchedulingPriority",
        "NtGdiDdDDIDestroyDCFromMemory",
        "NtGdiDdDDICreateDCFromMemory",
        "NtGdiDdDDIGetDeviceState",
        "NtGdiDdDDISetGammaRamp",
        "NtGdiDdDDIWaitForVerticalBlankEvent",
        "NtGdiDdDDIDestroyOverlay",
        "NtGdiDdDDIFlipOverlay",
        "NtGdiDdDDIUpdateOverlay",
        "NtGdiDdDDICreateOverlay",
        "NtGdiDdDDIGetPresentQueueEvent",
        "NtGdiDdDDIGetPresentHistory",
        "NtGdiDdDDISetVidPnSourceOwner1",
        "NtGdiDdDDISetVidPnSourceOwner",
        "NtGdiDdDDIQueryStatistics",
        "NtGdiDdDDIEscape",
        "NtGdiDdDDIGetSharedPrimaryHandle",
        "NtGdiDdDDICloseAdapter",
        "NtGdiDdDDIOpenAdapterFromLuid",
        "NtGdiDdDDIEnumAdapters",
        "NtGdiDdDDIEnumAdapters2",
        "NtGdiDdDDIOpenAdapterFromHdc",
        "NtGdiDdDDIOpenAdapterFromDeviceName",
        "NtGdiDdDDIRender",
        "NtGdiDdDDIPresent",
        "NtGdiDdDDIGetMultisampleMethodList",
        "NtGdiDdDDISetDisplayMode",
        "NtGdiDdDDIGetDisplayModeList",
        "NtGdiDdDDIUnlock",
        "NtGdiDdDDILock",
        "NtGdiDdDDIQueryAdapterInfo",
        "NtGdiDdDDIGetRuntimeData",
        "NtGdiDdDDISignalSynchronizationObject",
        "NtGdiDdDDIWaitForSynchronizationObject",
        "NtGdiDdDDIDestroySynchronizationObject",
        "NtGdiDdDDIOpenSynchronizationObject",
        "NtGdiDdDDICreateSynchronizationObject",
        "NtGdiDdDDIDestroyContext",
        "NtGdiDdDDICreateContext",
        "NtGdiDdDDIDestroyDevice",
        "NtGdiDdDDICreateDevice",
        "NtGdiDdDDIQueryAllocationResidency",
        "NtGdiDdDDISetAllocationPriority",
        "NtGdiDdDDIDestroyAllocation",
        "NtGdiDdDDIDestroyAllocation2",
        "NtGdiDdDDIOpenResourceFromNtHandle",
        "NtGdiDdDDIOpenSyncObjectFromNtHandle",
        "NtGdiDdDDIOpenSyncObjectFromNtHandle2",
        "NtGdiDdDDIOpenResource",
        "NtGdiDdDDIOpenNtHandleFromName",
        "NtGdiDdDDIOpenSyncObjectNtHandleFromName",
        "NtGdiDdDDIShareObjects",
        "NtGdiDdDDIQueryResourceInfoFromNtHandle",
        "NtGdiDdDDIQueryResourceInfo",
        "NtGdiDdDDICreateAllocation",
        "NtGdiDdDDIOutputDuplReleaseFrame",
        "NtGdiDdDDIQueryRemoteVidPnSourceFromGdiDisplayName",
        "NtGdiDdDDIOutputDuplPresent",
        "NtGdiDdDDIReleaseKeyedMutex2",
        "NtGdiDdDDIAcquireKeyedMutex2",
        "NtGdiDdDDIOpenKeyedMutex2",
        "NtGdiDdDDICreateKeyedMutex2",
        "NtGdiDdDDIOutputDuplGetPointerShapeData",
        "NtGdiDdDDIOutputDuplGetMetaData",
        "NtGdiDdDDIOutputDuplGetFrameInfo",
        "NtGdiDdDDIDestroyOutputDupl",
        "NtGdiDdDDICreateOutputDupl",
        "NtGdiDdDDIReclaimAllocations",
        "NtGdiDdDDIOfferAllocations",
        "NtGdiDdDDICheckSharedResourceAccess",
        "NtGdiDdDDICheckVidPnExclusiveOwnership",
        "NtGdiDdDDIGetOverlayState",
        "NtGdiDdDDIConfigureSharedResource",
        "NtGdiDdDDIReleaseKeyedMutex",
        "NtGdiDdDDIAcquireKeyedMutex",
        "NtGdiDdDDIDestroyKeyedMutex",
        "NtGdiDdDDIOpenKeyedMutex",
        "NtGdiDdDDICreateKeyedMutex",
        "NtGdiDdDDISharedPrimaryUnLockNotification",
        "NtGdiDdDDISharedPrimaryLockNotification",
        "NtGdiDdDDISetDisplayPrivateDriverFormat",
        "NtGdiDdDDICheckExclusiveOwnership",
        "NtGdiDdDDICheckMonitorPowerState",
        "NtGdiDdDDIWaitForIdle",
        "NtGdiDdDDICheckOcclusion",
        "NtGdiDdDDIInvalidateActiveVidPn",
        "NtGdiDdDDIPollDisplayChildren",
        "NtGdiDdDDISetQueuedLimit",
        "NtGdiDdDDIPinDirectFlipResources",
        "NtGdiDdDDIUnpinDirectFlipResources",
        "NtGdiDdDDIWaitForVerticalBlankEvent2",
        "NtGdiDdDDIGetDWMVerticalBlankEvent",
        "NtGdiDdDDISetSyncRefreshCountWaitTarget",
        "NtGdiDdDDISetContextInProcessSchedulingPriority",
        "NtGdiDdDDIGetContextInProcessSchedulingPriority",
        "NtGdiDdDDIGetSharedResourceAdapterLuid",
        "NtGdiDdDDISetStereoEnabled",
        "NtGdiDdDDIPresentMultiPlaneOverlay",
        "NtGdiDdDDICheckMultiPlaneOverlaySupport",
        "NtGdiDdDDIMakeResident",
        "NtGdiDdDDIEvict",
        "NtGdiDdDDIUpdateAllocationProperty",
        "NtGdiDdDDIWaitForSynchronizationObjectFromCpu",
        "NtGdiDdDDISignalSynchronizationObjectFromCpu",
        "NtGdiDdDDIWaitForSynchronizationObjectFromGpu",
        "NtGdiDdDDISignalSynchronizationObjectFromGpu",
        "NtGdiDdDDISignalSynchronizationObjectFromGpu2",
        "NtGdiDdDDICreatePagingQueue",
        "NtGdiDdDDIDestroyPagingQueue",
        "NtGdiDdDDILock2",
        "NtGdiDdDDIUnlock2",
        "NtGdiDdDDIInvalidateCache",
        "NtGdiDdDDIGetResourcePresentPrivateDriverData",
        "NtGdiDdDDIMapGpuVirtualAddress",
        "NtGdiDdDDIReserveGpuVirtualAddress",
        "NtGdiDdDDIFreeGpuVirtualAddress",
        "NtGdiDdDDIUpdateGpuVirtualAddress",
        "NtGdiDdDDICreateContextVirtual",
        "NtGdiDdDDISubmitCommand",
        "NtGdiDdDDIGetCachedHybridQueryValue",
        "NtGdiDdDDICacheHybridQueryValue",
        "NtGdiDdDDINetDispGetNextChunkInfo",
        "NtGdiDdDDINetDispQueryMiracastDisplayDeviceSupport",
        "NtGdiDdDDINetDispStartMiracastDisplayDevice",
        "NtGdiDdDDINetDispStopMiracastDisplayDevice",
        "NtGdiDdDDINetDispQueryMiracastDisplayDeviceStatus",
        "NtGdiDdDDINetDispStopSessions",
        "NtGdiDdDDIQueryVideoMemoryInfo",
        "NtGdiDdDDIChangeVideoMemoryReservation",
        "NtGdiDdDDICreateSwapChain",
        "NtGdiDdDDIOpenSwapChain",
        "NtGdiDdDDIGetSetSwapChainMetadata",
        "NtGdiDdDDIAcquireSwapChain",
        "NtGdiDdDDIReleaseSwapChain",
        "NtGdiDdDDIAbandonSwapChain",
        "NtGdiDdDDISetDodIndirectSwapchain",
        "NtGdiDdDDICheckMultiPlaneOverlaySupport2",
        "NtGdiDdDDIPresentMultiPlaneOverlay2",
        "NtGdiDdDDIReclaimAllocations2",
        "NtGdiDdDDISetStablePowerState",
        "NtGdiDdDDIQueryClockCalibration",
        "NtGdiDdDDIQueryVidPnExclusiveOwnership",
        "NtGdiDdDDIAdjustFullscreenGamma",
        "NtGdiDdDDISetVidPnSourceHwProtection",
        "NtGdiDdDDIMarkDeviceAsError",
        "NtGdiDdDDIFlushHeapTransitions",
        "NtGdiDdDDISetHwProtectionTeardownRecovery",
        "NtGdiDdDDIQueryProcessOfferInfo",
        "NtGdiDdDDITrimProcessCommitment",
        "NtGdiDdDDICheckMultiPlaneOverlaySupport3",
        "NtGdiDdDDIPresentMultiPlaneOverlay3",
        "NtGdiDdDDISetFSEBlock",
        "NtGdiDdDDIQueryFSEBlock",
        "NtGdiDdDDICreateHwContext",
        "NtGdiDdDDIDestroyHwContext",
        "NtGdiDdDDICreateHwQueue",
        "NtGdiDdDDIDestroyHwQueue",
        "NtGdiDdDDISubmitCommandToHwQueue",
        "NtGdiDdDDISubmitWaitForSyncObjectsToHwQueue",
        "NtGdiDdDDISubmitSignalSyncObjectsToHwQueue",
        "NtGdiDdDDIGetAllocationPriority",
        "NtGdiDdDDIGetMultiPlaneOverlayCaps",
        "NtGdiDdDDIGetPostCompositionCaps",
        "NtGdiDdDDISetYieldPercentage",
        "NtGdiDdDDISetProcessSchedulingPriorityBand",
        "NtGdiDdDDISetMemoryBudgetTarget",
        "NtGdiDdDDIGetYieldPercentage",
        "NtGdiDdDDIGetProcessSchedulingPriorityBand",
        "NtGdiDdDDIGetMemoryBudgetTarget",
        "NtGdiMakeObjectUnXferable",
        "NtGdiMakeObjectXferable",
        "NtGdiDestroyPhysicalMonitor",
        "NtGdiGetPhysicalMonitorDescription",
        "NtGdiGetPhysicalMonitors",
        "NtGdiGetNumberOfPhysicalMonitors",
        "NtGdiDDCCIGetTimingReport",
        "NtGdiDDCCIGetCapabilitiesString",
        "NtGdiDDCCIGetCapabilitiesStringLength",
        "NtGdiDDCCISaveCurrentSettings",
        "NtGdiDDCCISetVCPFeature",
        "NtGdiDDCCIGetVCPFeature",
        "NtGdiDdQueryVisRgnUniqueness",
        "NtGdiDdDestroyFullscreenSprite",
        "NtGdiDdNotifyFullscreenSpriteUpdate",
        "NtGdiDdCreateFullscreenSprite",
        "NtGdiGetProcessSessionFonts",
        "NtGdiGetPublicFontTableChangeCookie",
        "NtGdiAddInitialFonts",
        "NtUserShowSystemCursor",
        "NtUserSetMirrorRendering",
        "NtUserMagGetContextInformation",
        "NtUserMagSetContextInformation",
        "NtUserMagControl",
        "NtUserSlicerControl",
        "NtUserHwndSetRedirectionInfo",
        "NtUserHwndQueryRedirectionInfo",
        "NtCreateCompositionSurfaceHandle",
        "NtValidateCompositionSurfaceHandle",
        "NtBindCompositionSurface",
        "NtUnBindCompositionSurface",
        "NtQueryCompositionSurfaceBinding",
        "NtNotifyPresentToCompositionSurface",
        "NtQueryCompositionSurfaceStatistics",
        "NtOpenCompositionSurfaceSectionInfo",
        "NtOpenCompositionSurfaceSwapChainHandleInfo",
        "NtQueryCompositionSurfaceRenderingRealization",
        "NtOpenCompositionSurfaceDirtyRegion",
        "NtQueryCompositionSurfaceHDRMetaData",
        "NtSetCompositionSurfaceHDRMetaData",
        "NtSetCompositionSurfaceDirectFlipState",
        "NtSetCompositionSurfaceStatistics",
        "NtSetCompositionSurfaceBufferUsage",
        "NtSetCompositionSurfaceIndependentFlipInfo",
        "NtDesktopCaptureBits",
        "NtDCompositionEnableMMCSS",
        "NtVisualCaptureBits",
        "NtDCompositionEnableDDASupport",
        "NtDCompositionCreateSharedVisualHandle",
        "NtCreateCompositionInputSink",
        "NtCreateImplicitCompositionInputSink",
        "NtDuplicateCompositionInputSink",
        "NtQueryCompositionInputSink",
        "NtQueryCompositionInputSinkLuid",
        "NtQueryCompositionInputSinkViewId",
        "NtUpdateInputSinkTransforms",
        "NtCompositionInputThread",
        "NtQueryCompositionInputQueueAndTransform",
        "NtQueryCompositionInputIsImplicit",
        "NtCompositionSetDropTarget",
        "NtTokenManagerOpenSectionAndEvents",
        "NtTokenManagerThread",
        "NtTokenManagerCreateCompositionTokenHandle",
        "NtTokenManagerGetAnalogExclusiveTokenEvent",
        "NtTokenManagerGetAnalogExclusiveSurfaceUpdates",
        "NtTokenManagerConfirmOutstandingAnalogToken",
        "NtSetCompositionSurfaceAnalogExclusive",
        "NtDCompositionBeginFrame",
        "NtDCompositionConfirmFrame",
        "NtDCompositionRetireFrame",
        "NtDCompositionDiscardFrame",
        "NtDCompositionGetFrameSurfaceUpdates",
        "NtDCompositionGetFrameLegacyTokens",
        "NtDCompositionDestroyConnection",
        "NtDCompositionGetConnectionBatch",
        "NtDCompositionGetFrameStatistics",
        "NtDCompositionGetDeletedResources",
        "NtDCompositionCreateConnection",
        "NtDCompositionDestroyChannel",
        "NtDCompositionReleaseAllResources",
        "NtDCompositionSubmitDWMBatch",
        "NtDCompositionProcessChannelBatchBuffer",
        "NtDCompositionCommitChannel",
        "NtDCompositionWaitForChannel",
        "NtDCompositionSetChannelCommitCompletionEvent",
        "NtDCompositionTelemetryTouchInteractionBegin",
        "NtDCompositionTelemetryTouchInteractionUpdate",
        "NtDCompositionTelemetryTouchInteractionEnd",
        "NtDCompositionTelemetrySetApplicationId",
        "NtDCompositionTelemetryAnimationScenarioBegin",
        "NtDCompositionTelemetryAnimationScenarioReference",
        "NtDCompositionTelemetryAnimationScenarioUnreference",
        "NtDCompositionCurrentBatchId",
        "NtDCompositionRemoveCrossDeviceVisualChild",
        "NtDCompositionAddCrossDeviceVisualChild",
        "NtDCompositionCreateDwmChannel",
        "NtDCompositionCreateChannel",
        "NtDCompositionSynchronize",
        "NtDCompositionReferenceSharedResourceOnDwmChannel",
        "NtDCompositionCreateAndBindSharedSection",
        "NtDCompositionSetDebugCounter",
        "NtDCompositionGetChannels",
        "NtDCompositionConnectPipe",
        "NtDCompositionRegisterThumbnailVisual",
        "NtDCompositionRegisterVirtualDesktopVisual",
        "NtDCompositionDuplicateHandleToProcess",
        "NtDCompositionUpdatePointerCapture",
        "NtDCompositionSetChannelCallbackId",
        "NtDCompositionDuplicateSwapchainHandleToDwm",
        "NtDCompositionSetChildRootVisual",
        "NtDCompositionCommitSynchronizationObject",
        "NtUserDestroyDCompositionHwndTarget",
        "NtUserCreateDCompositionHwndTarget",
        "NtUserWaitForRedirectionStartComplete",
        "NtUserSignalRedirectionStartComplete",
        "NtUserEnableWindowResizeOptimization",
        "NtUserGetResizeDCompositionSynchronizationObject",
        "NtUserEnableResizeLayoutSynchronization",
        "NtUserBeginLayoutUpdate",
        "NtUserIsResizeLayoutSynchronizationEnabled",
        "NtUserConfirmResizeCommit",
        "NtUserSetActiveProcessForMonitor",
        "NtUserGetDisplayAutoRotationPreferencesByProcessId",
        "NtUserGetDisplayAutoRotationPreferences",
        "NtUserSetDisplayAutoRotationPreferences",
        "NtUserSetAutoRotation",
        "NtUserGetAutoRotationState",
        "NtUserAutoRotateScreen",
        "NtUserAcquireIAMKey",
        "NtUserSetActivationFilter",
        "NtUserSetFallbackForeground",
        "NtUserSetBrokeredForeground",
        "NtUserDisableImmersiveOwner",
        "NtUserClearForeground",
        "NtUserEnableIAMAccess",
        "NtUserGetProcessUIContextInformation",
        "NtUserSetProcessRestrictionExemption",
        "NtUserSetWindowArrangement",
        "NtUserSetWindowShowState",
        "NtUserUpdateWindowTrackingInfo",
        "NtUserEnableMouseInPointer",
        "NtUserIsMouseInPointerEnabled",
        "NtUserPromoteMouseInPointer",
        "NtUserAutoPromoteMouseInPointer",
        "NtUserEnableMouseInputForCursorSuppression",
        "NtUserIsMouseInputEnabled",
        "NtUserCheckProcessForClipboardAccess",
        "NtUserGetClipboardAccessToken",
        "NtUserGetQueueStatusReadonly",
        "NtUserCompositionInputSinkLuidFromPoint",
        "NtUserCompositionInputSinkViewInstanceIdFromPoint",
        "NtUserUpdateWindowInputSinkHints",
        "NtUserTransformPoint",
        "NtUserTransformRect",
        "NtUserGetHimetricScaleFactorFromPixelLocation",
        "NtUserGetDpiForMonitor",
        "NtUserReportInertia",
        "NtUserLinkDpiCursor",
        "NtUserGetCursorDims",
        "NtUserInitializeInputDeviceInjection",
        "NtUserInitializeGenericHidInjection",
        "NtUserInitializePointerDeviceInjection",
        "NtUserRemoveInjectionDevice",
        "NtUserSetFeatureReportResponse",
        "NtUserInjectDeviceInput",
        "NtUserInjectMouseInput",
        "NtUserInjectKeyboardInput",
        "NtUserInjectPointerInput",
        "NtUserInjectGenericHidInput",
        "NtUserInitializePointerDeviceInjectionEx",
        "NtRIMRegisterForInput",
        "NtRIMReadInput",
        "NtRIMGetDevicePreparsedData",
        "NtRIMGetDeviceProperties",
        "NtRIMAreSiblingDevices",
        "NtRIMFreeInputBuffer",
        "NtRIMOnPnpNotification",
        "NtRIMOnTimerNotification",
        "NtRIMDeviceIoControl",
        "NtRIMUnregisterForInput",
        "NtRIMSetTestModeStatus",
        "NtRIMGetPhysicalDeviceRect",
        "NtRIMGetSourceProcessId",
        "NtRIMAddInputObserver",
        "NtRIMRemoveInputObserver",
        "NtRIMUpdateInputObserverRegistration",
        "NtRIMObserveNextInput",
        "NtRIMGetDevicePreparsedDataLockfree",
        "NtRIMGetDevicePropertiesLockfree",
        "NtUserSetCoreWindow",
        "NtUserSetCoreWindowPartner",
        "NtUserNavigateFocus",
        "NtHWCursorUpdatePointer",
        "NtUserAcquireInteractiveControlBackgroundAccess",
        "NtUserGetInteractiveControlInfo",
        "NtUserGetInteractiveControlDeviceInfo",
        "NtUserSendInteractiveControlHapticsReport",
        "NtUserSetInteractiveControlFocus",
        "NtUserInteractiveControlQueryUsage",
        "NtUserSetInteractiveCtrlRotationAngle",
        "NtUserGetInteractiveCtrlSupportedWaveforms",
        "NtUserProcessInkFeedbackCommand",
        "NtUserSetProcessInteractionFlags",
        "NtMITActivateInputProcessing",
        "NtMITWaitForMultipleObjectsEx",
        "NtMITDeactivateInputProcessing",
        "NtMITSetInputCallbacks",
        "NtMITCoreMsgKGetConnectionHandle",
        "NtMITCoreMsgKSend",
        "NtMITCoreMsgKOpenConnectionTo",
        "NtMITUpdateInputGlobals",
        "NtMITBindInputTypeToMonitors",
        "NtMITEnableMouseIntercept",
        "NtMITDisableMouseIntercept",
        "NtMITSynthesizeTouchInput",
        "NtMITSynthesizeMouseInput",
        "NtMITSynthesizeMouseWheel"
    ]
]


# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""
import struct

import volatility.obj as obj
import volatility.utils as utils
import volatility.poolscan as poolscan
import volatility.debug as debug

import volatility.plugins.linux.common as linux_common
import volatility.plugins.linux.pslist as pslist

from volatility.renderers import TreeGrid
from volatility.renderers.basic import Address

class linux_psscan(pslist.linux_pslist):
    """ Scan physical memory for processes """

    def __init__(self, config, *args, **kwargs):
        linux_common.AbstractLinuxCommand.__init__(self, config, *args, **kwargs)
        self.wants_physical = True

    def calculate(self):
        linux_common.set_plugin_members(self)
        
        phys_addr_space = utils.load_as(self._config, astype = 'physical')

        if phys_addr_space.profile.metadata.get('memory_model', '32bit') == "32bit":
            fmt  = "<I"
        else:
            fmt  = "<Q"

        needles     = []
        
        for sym in phys_addr_space.profile.get_all_symbol_names("kernel"):
            if sym.find("_sched_class") != -1:
                addr = phys_addr_space.profile.get_symbol(sym)
                needles.append(struct.pack(fmt, addr)) 

        if len(needles) == 0:
            debug.warning("Unable to scan for processes. Please file a bug report.")
        else:
            back_offset = phys_addr_space.profile.get_obj_offset("task_struct", "sched_class")

            scanner = poolscan.MultiPoolScanner(needles)    

            for _, offset in scanner.scan(phys_addr_space):
                ptask = obj.Object("task_struct", offset = offset - back_offset, vm = phys_addr_space)

                if not ptask.exit_state.v() in [0, 16, 32, 16|32]:
                    continue

                if not (0 < ptask.pid < 66000):
                    continue

                yield ptask

            



#
# parse_pbzx.py 
#  Useful for extracting "Payload" files from newer Kernel Debug Kits
#   you can then decompress that with cpio -i < Payload.part00.cpio.xz
#
# Taken from https://gist.github.com/pudquick/ff412bcb29c9c1fa4b8d
# 
# Original notes:
#
# v2 pbzx stream handler
# My personal writeup on the differences here: https://gist.github.com/pudquick/29fcfe09c326a9b96cf5
#
# Pure python reimplementation of .cpio.xz content extraction from pbzx file payload originally here:
# http://www.tonymacx86.com/general-help/135458-pbzx-stream-parser.html
#
# Cleaned up C version (as the basis for my code) here, thanks to Pepijn Bruienne / @bruienne
# https://gist.github.com/bruienne/029494bbcfb358098b41

import struct, sys

def seekread(f, offset=None, length=0, relative=True):
    if (offset != None):
        # offset provided, let's seek
        f.seek(offset, [0,1,2][relative])
    if (length != 0):
        return f.read(length)

def parse_pbzx(pbzx_path):
    section = 0
    xar_out_path = '%s.part%02d.cpio.xz' % (pbzx_path, section)
    f = open(pbzx_path, 'rb')
    # pbzx = f.read()
    # f.close()
    magic = seekread(f,length=4)
    if magic != 'pbzx':
        raise "Error: Not a pbzx file"
    # Read 8 bytes for initial flags
    flags = seekread(f,length=8)
    # Interpret the flags as a 64-bit big-endian unsigned int
    flags = struct.unpack('>Q', flags)[0]
    xar_f = open(xar_out_path, 'wb')
    while (flags & (1 << 24)):
        # Read in more flags
        flags = seekread(f,length=8)
        flags = struct.unpack('>Q', flags)[0]
        # Read in length
        f_length = seekread(f,length=8)
        f_length = struct.unpack('>Q', f_length)[0]
        xzmagic = seekread(f,length=6)
        if xzmagic != '\xfd7zXZ\x00':
            # This isn't xz content, this is actually _raw decompressed cpio_ chunk of 16MB in size...
            # Let's back up ...
            seekread(f,offset=-6,length=0)
            # ... and split it out ...
            f_content = seekread(f,length=f_length)
            section += 1
            decomp_out = '%s.part%02d.cpio' % (pbzx_path, section)
            g = open(decomp_out, 'wb')
            g.write(f_content)
            g.close()
            # Now to start the next section, which should hopefully be .xz (we'll just assume it is ...)
            xar_f.close()
            section += 1
            new_out = '%s.part%02d.cpio.xz' % (pbzx_path, section)
            xar_f = open(new_out, 'wb')
        else:
            f_length -= 6
            # This part needs buffering
            f_content = seekread(f,length=f_length)
            tail = seekread(f,offset=-2,length=2)
            xar_f.write(xzmagic)
            xar_f.write(f_content)
            if tail != 'YZ':
                xar_f.close()
                raise "Error: Footer is not xar file footer"
    try:
        f.close()
        xar_f.close()
    except:
        pass

def main():
    result = parse_pbzx(sys.argv[1])
    print "Now xz decompress the .xz chunks, then 'cat' them all together in order into a single new.cpio file"
 
if __name__ == '__main__':
    main()



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""
import volatility.obj   as obj
import volatility.utils as utils
import volatility.debug as debug

import volatility.plugins.mac.common as common

class mac_check_fop(common.AbstractMacCommand):
    """ Validate File Operation Pointers """

    def _walk_vfstbllist(self, kaddr_info):
        table_size_ptr = self.addr_space.profile.get_symbol("_maxvfsconf")

        if table_size_ptr == None:
            table_size_ptr = self.addr_space.profile.get_symbol("_maxvfsslots")

        table_size = obj.Object("unsigned int", offset = table_size_ptr, vm = self.addr_space)

        table_ptr = self.addr_space.profile.get_symbol("_vfstbllist")
        table = obj.Object(theType = "Array", targetType = "vfstable", offset = table_ptr, count = table_size, vm = self.addr_space) 
        vfs_op_members = self.profile.types['vfsops'].keywords["members"].keys()
        
        if "vfs_reserved" in vfs_op_members:
            vfs_op_members.remove("vfs_reserved")

        for vfs in table:
            if not vfs.is_valid():
                continue

            name = self.addr_space.read(vfs.vfc_name.obj_offset, 16)
            if name:
                idx = name.find("\x00")
                if idx != -1:
                    name = name[:idx]
            else:
                name = "<INVALID NAME>"

            if name == "<unassigned>":
                break

            ops = vfs.vfc_vfsops

            for member in vfs_op_members:
                ptr = ops.__getattr__(member).v()

                if ptr == 0:
                    continue
                
                (module, handler_sym) = common.get_handler_name(kaddr_info, ptr)

                yield (vfs.v(), name, ptr, module, handler_sym)

    def _walk_opv_desc(self, kaddr_info):
        table_addr = self.addr_space.profile.get_symbol("_vfs_opv_descs")

        table = obj.Object(targetType = "unsigned long", theType = "Array", count = 32, vm = self.addr_space, offset = table_addr)

        for desc in table:
            if desc.v() == 0:
                break

            table_name = self.addr_space.profile.get_symbol_by_address("kernel", desc.v())
            if not table_name:
                table_name = "<unknown table>"

            vnodeopv_desc = obj.Object("vnodeopv_desc", offset = desc.v(), vm = self.addr_space)

            vdesc_arr = obj.Object(theType = "Array", targetType = "vnodeopv_entry_desc", offset = vnodeopv_desc.opv_desc_ops, count = 64, vm = self.addr_space)

            for vdesc in vdesc_arr: 
                ptr = vdesc.opve_impl.v()
                if ptr == 0:
                    break

                name = self.addr_space.read(vdesc.opve_op.vdesc_name.v(), 64)
                if name:
                    idx = name.find("\x00")
                    if idx != -1:
                        name = name[:idx]
                else:
                    name = "<INVALID NAME>"
                    
                name = table_name + "/" + name

                (module, handler_sym) = common.get_handler_name(kaddr_info, ptr)

                yield (vdesc.v(), name, ptr, module, handler_sym)  

    def calculate(self):
        common.set_plugin_members(self)
        
        kaddr_info = common.get_handler_name_addrs(self)

        funcs = [self._walk_opv_desc, self._walk_vfstbllist]

        for func in funcs:
            for (vfs_ptr, name, ptr, module, handler_sym) in func(kaddr_info):
                yield (vfs_ptr, name, ptr, module, handler_sym) 
    
    def render_text(self, outfd, data):
        self.table_header(outfd, [("Offset", "[addrpad]"),
                          ("Name", "48"),
                          ("Handler", "[addrpad]"),
                          ("Module", "32"),
                          ("Handler Sym", "")])

        for (vfs_addr, name, handler, module, handler_sym) in data:
            self.table_row(outfd, vfs_addr, name, handler, module, handler_sym)




# Volatility EditBox plugin
#
# Author: Bridgey the Geek <bridgeythegeek@gmail.com>
#
# This plugin is free software; you can redistribute it and/or modify
# it under the terms of GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This plugin is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PRACTICAL PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this plugin. If not, see <http://www.gnu.org/licenses/>.
#
# This work heavily inspired by GDI Utilities from Dr Brendan Dolan-Gavitt PhD.
# <http://www.cc.gatech.edu/~brendan/volatility/>
#
# The iteration of the Windows objects is borrowed from the Windows plugin.
# <https://github.com/volatilityfoundation/volatility/blob/master/volatility/plugins/gui/windows.py>
#
# This plugin wouldn't exist without the assistance of those on the volusers
# mailing list <http://lists.volatilesystems.com/mailman/listinfo/vol-users>.

"""
@author     : Bridgey the Geek
@license    : GPL 2 or later
@contact    : bridgeythegeek@gmail.com
"""

import os

import volatility.debug as debug
import volatility.obj as obj
import volatility.utils as utils

import volatility.plugins.common as common
import volatility.plugins.gui.messagehooks as messagehooks
import volatility.win32 as win32
from volatility.renderers import TreeGrid

supported_controls = {
    'edit'   : 'COMCTL_EDIT',
    'listbox': 'COMCTL_LISTBOX',
}

editbox_vtypes_xp_x86 = {
    'COMCTL_EDIT': [0xEE, {
        'hBuf': [0x00, ['unsigned long']],
        'hWnd': [0x38, ['unsigned long']],
        'parenthWnd': [0x58, ['unsigned long']],
        'nChars': [0x0C, ['unsigned long']],
        'selStart': [0x14, ['unsigned long']],
        'selEnd': [0x18, ['unsigned long']],
        'pwdChar': [0x30, ['unsigned short']],
        'undoBuf': [0x80, ['unsigned long']],
        'undoPos': [0x84, ['long']],
        'undoLen': [0x88, ['long']],
        'bEncKey': [0xEC, ['unsigned char']],
    }],
    'COMCTL_LISTBOX': [0x40, {
        'hWnd': [0x00, ['unsigned long']],
        'parenthWnd': [0x04, ['unsigned long']],
        'atomHandle': [0x08, ['unsigned long']],
        'firstVisibleRow': [0x10, ['unsigned long']],
        'caretPos': [0x14, ['long']],
        'rowsVisible': [0x1C, ['unsigned long']],
        'itemCount': [0x20, ['unsigned long']],
        'stringsStart': [0x2C, ['unsigned long']],
        'stringsLength': [0x34, ['unsigned long']]
    }],
}

editbox_vtypes_xp_x64 = {
    'COMCTL_EDIT': [ 0x142, {
        'hBuf': [0x00, ['unsigned long']],
        'hWnd': [0x40, ['unsigned long']],
        'parenthWnd': [0x60, ['unsigned long']],
        'nChars': [0x10, ['unsigned long']],
        'selStart': [0x18, ['unsigned long']],
        'selEnd': [0x20, ['unsigned long']],
        'pwdChar': [0x34, ['unsigned short']],
        'undoBuf': [0xA8, ['address']],
        'undoPos': [0xB0, ['long']],
        'undoLen': [0xB4, ['long']],
        'bEncKey': [0x140, ['unsigned char']]
    }],
    'COMCTL_LISTBOX': [ 0x100, {
        'hWnd': [0x00, ['unsigned long']],
        'parenthWnd': [0x08, ['unsigned long']],
        'firstVisibleRow': [0x20, ['unsigned long']],
        'caretPos': [0x28, ['unsigned long']],
        'rowsVisible': [0x2C, ['unsigned long']],
        'itemCount': [0x30, ['unsigned long']],
        'stringsStart': [0x40, ['address']],
        'stringsLength': [0x4C, ['unsigned long']]
    }],
}

editbox_vtypes_vista7810_x86 = {
    'COMCTL_EDIT': [0xF6, {
        'hBuf': [0x00, ['unsigned long']],
        'hWnd': [0x38, ['unsigned long']],
        'parenthWnd': [0x58, ['unsigned long']],
        'nChars': [0x0C, ['unsigned long']],
        'selStart': [0x14, ['unsigned long']],
        'selEnd': [0x18, ['unsigned long']],
        'pwdChar': [0x30, ['unsigned short']],
        'undoBuf': [0x88, ['unsigned long']],
        'undoPos': [0x8C, ['long']],
        'undoLen': [0x90, ['long']],
        'bEncKey': [0xF4, ['unsigned char']],
    }],
    'COMCTL_LISTBOX': [0x40, {
        'hWnd': [0x00, ['unsigned long']],
        'parenthWnd': [0x04, ['unsigned long']],
        'atomHandle': [0x08, ['unsigned long']],
        'firstVisibleRow': [0x10, ['unsigned long']],
        'caretPos': [0x14, ['long']],
        'rowsVisible': [0x1C, ['unsigned long']],
        'itemCount': [0x20, ['unsigned long']],
        'stringsStart': [0x2C, ['unsigned long']],
        'stringsLength': [0x34, ['unsigned long']]
    }],
}

editbox_vtypes_vista7810_x64 = {
    'COMCTL_EDIT': [0x142, {
        'hBuf': [0x00, ['unsigned long']],
        'hWnd': [0x40, ['unsigned long']],
        'parenthWnd': [0x60, ['unsigned long']],
        'nChars': [0x10, ['unsigned long']],
        'selStart': [0x18, ['unsigned long']],
        'selEnd': [0x20, ['unsigned long']],
        'pwdChar': [0x34, ['unsigned short']],
        'undoBuf': [0xA8, ['address']],
        'undoPos': [0xB0, ['long']],
        'undoLen': [0xB4, ['long']],
        'bEncKey': [0x140, ['unsigned char']],
    }],
    'COMCTL_LISTBOX': [0x54, {
        'hWnd': [0x00, ['unsigned long']],
        'parenthWnd': [0x08, ['unsigned long']],
        'firstVisibleRow': [0x20, ['unsigned long']],
        'caretPos': [0x28, ['unsigned long']],
        'rowsVisible': [0x2C, ['unsigned long']],
        'itemCount': [0x30, ['unsigned long']],
        'stringsStart': [0x40, ['address']],
        'stringsLength': [0x4C, ['unsigned long']]
    }],
}


class COMCTL_EDIT(obj.CType):
    """Methods for the Edit structure"""

    def __str__(self):
        """String representation of the Edit"""

        _MAX_OUT = 50

        text = self.get_text(no_crlf=True)
        text = '{}...'.format(text[:_MAX_OUT - 3]) if len(text) > _MAX_OUT else text

        undo = self.get_undo(no_crlf=True)
        undo = '{}...'.format(undo[:_MAX_OUT - 3]) if len(undo) > _MAX_OUT else undo

        return '<{0}(Text="{1}", Len={2}, Pwd={3}, Undo="{4}", UndoLen={5})>'.format(
            self.__class__.__name__, text, self.nChars, self.is_pwd(), undo, self.undoLen)

    def get_text(self, no_crlf=False):
        """Get the text from the control

        :param no_crlf:
        :return:
        """

        if self.nChars < 1:
            return ''
        text_deref = obj.Object('unsigned long', offset=self.hBuf, vm=self.obj_vm)
        raw = self.obj_vm.read(text_deref, self.nChars * 2)
        if not self.pwdChar == 0x00:  # Is a password dialog
            raw = COMCTL_EDIT.rtl_run_decode_unicode_string(self.bEncKey, raw)
        if no_crlf:
            return raw.decode('utf-16').replace('\r\n', '.')
        else:
            return raw.decode('utf-16')

    def get_undo(self, no_crlf=False):
        """Get the contents of the undo buffer

        :param no_crlf:
        :return:
        """

        if self.undoLen < 1:
            return ''
        if no_crlf:
            return self.obj_vm.read(self.undoBuf, self.undoLen * 2).decode('utf-16').replace('\r\n', '.')
        else:
            return self.obj_vm.read(self.undoBuf, self.undoLen * 2).decode('utf-16')

    def is_pwd(self):
        """Is this a password control?

        :return:
        """

        return self.pwdChar != 0x00

    def dump_meta(self, outfd):
        """Dumps the meta data of the control
        
        @param  outfd: 
        """
        outfd.write('nChars            : {}\n'.format(self.nChars))
        outfd.write('selStart          : {}\n'.format(self.selStart))
        outfd.write('selEnd            : {}\n'.format(self.selEnd))
        outfd.write('isPwdControl      : {}\n'.format(self.is_pwd()))
        outfd.write('undoPos           : {}\n'.format(self.undoPos))
        outfd.write('undoLen           : {}\n'.format(self.undoLen))
        outfd.write('address-of undoBuf: {:#x}\n'.format(self.undoBuf))
        outfd.write('undoBuf           : {}\n'.format(self.get_undo(no_crlf=True)))

    def dump_data(self, outfd):
        """Dumps the data of the control
        
        @param  outfd: 
        """
        outfd.write('{}\n'.format(self.get_text()))

    @staticmethod
    def rtl_run_decode_unicode_string(key, data):
        s = ''.join([chr(ord(data[i - 1]) ^ ord(data[i]) ^ key) for i in range(1, len(data))])
        s = chr(ord(data[0]) ^ (key | 0x43)) + s
        return s


class COMCTL_LISTBOX(obj.CType):
    """Methods for the Listbox structure"""

    def __str__(self):
        """String representation of the Listbox"""

        _MAX_OUT = 50

        text = self.get_text(joiner='|')
        text = '{}...'.format(text[:_MAX_OUT - 3]) if len(text) > _MAX_OUT else text

        return '<{0}(Text="{1}", Items={2}, Caret={3}>'.format(
            self.__class__.__name__, text, self.itemCount, self.caretPos)
    
    def get_text(self, joiner='\n'):
        """Get the text from the control

        @param joiner:
        @return:
        """

        if self.stringsLength < 1:
            return ''
        raw = self.obj_vm.read(self.stringsStart, self.stringsLength)
        return joiner.join(split_null_strings(raw))

    def dump_meta(self, outfd):
        """Dumps the meta data of the control

        @param  outfd:
        """

        outfd.write('firstVisibleRow   : {}\n'.format(self.firstVisibleRow))
        outfd.write('caretPos          : {}\n'.format(self.caretPos))
        outfd.write('rowsVisible       : {}\n'.format(self.rowsVisible))
        outfd.write('itemCount         : {}\n'.format(self.itemCount))
        outfd.write('stringsStart      : {:#x}\n'.format(self.stringsStart))
        outfd.write('stringsLength     : {}\n'.format(self.stringsLength))

    def dump_data(self, outfd):
        """Dumps the data of the control

        @param  outfd:
        """

        outfd.write('{}\n'.format(self.get_text()))


def split_null_strings(data):
    """Splits a concatenation of null-terminated utf-16 strings
    
    @param  data:
    """
    
    strings = []
    start = 0
    for i in xrange(0, len(data), 2):
        if data[i] == '\x00' and data[i+1] == '\x00':
            strings.append(data[start:i])
            start = i+2
    return [s.decode('utf-16') for s in strings]

def dump_to_file(ctrl, pid, proc_name, folder):
    """Dumps the data of the control to a file

    @param  ctrl:
    @param  pid:
    @param  proc_name:
    @param  folder:
    """
    ctrl_safe_name = str(ctrl.__class__.__name__).split('_')[-1].lower()
    file_name = '{0}_{1}_{2}_{3:#x}.txt'.format(pid, proc_name, ctrl_safe_name, ctrl.v())
    with open(os.path.join(folder, file_name), 'wb') as out_file:
        out_file.write(ctrl.get_text())


class Editbox(common.AbstractWindowsCommand):
    """Displays information about Edit controls. (Listbox experimental.)"""

    # Add the classes for the structures
    editbox_classes = {
        'COMCTL_EDIT'   : COMCTL_EDIT,
        'COMCTL_LISTBOX': COMCTL_LISTBOX,
    }

    def __init__(self, config, *args, **kwargs):
        common.AbstractWindowsCommand.__init__(self, config, *args, **kwargs)

        # Filter specific processes
        config.add_option('PID', short_option='p', default=None,
                          help='Operate on these Process IDs (comma-separated)',
                          action='store', type='str')
        config.add_option('DUMP-DIR', short_option='D', default=None,
                          help='Save the found text to files in this folder',
                          action='store', type='str')
        
        self.fake_32bit = False

    @staticmethod
    def apply_types(addr_space, meta=None):
        """Add the correct vtypes and classes for the profile

        @param  addr_space:        
        @param  meta: 
        """

        if not meta:
            meta = addr_space.profile.metadata
        
        if meta['os'] == 'windows':
            if meta['major'] == 5:
                if meta['memory_model'] == '32bit':
                    addr_space.profile.vtypes.update(editbox_vtypes_xp_x86)
                elif meta['memory_model'] == '64bit':
                    addr_space.profile.vtypes.update(editbox_vtypes_xp_x64)
                else:
                    debug.error("The selected address space is not supported")
                addr_space.profile.compile()
            elif meta['major'] == 6:
                if meta['memory_model'] == '32bit':
                    addr_space.profile.vtypes.update(editbox_vtypes_vista7810_x86)
                elif meta['memory_model'] == '64bit':
                    addr_space.profile.vtypes.update(editbox_vtypes_vista7810_x64)
                else:
                    debug.error("The selected address space is not supported")
                addr_space.profile.compile()
            else:
                debug.error("The selected address space is not supported")
        else:
            debug.error("The selected address space is not supported")

    def calculate(self):
        """Parse the control structures"""

        # Check the output folder exists
        if self._config.DUMP_DIR and not os.path.isdir(self._config.dump_dir):
            debug.error('{0} is not a directory'.format(self._config.dump_dir))

        # Apply the correct vtypes for the profile
        addr_space = utils.load_as(self._config)
        addr_space.profile.object_classes.update(Editbox.editbox_classes)
        self.apply_types(addr_space)

        # Build a list of tasks
        tasks = win32.tasks.pslist(addr_space)
        if self._config.PID:
            pids = [int(p) for p in self._config.PID.split(',')]
            the_tasks = [t for t in tasks if t.UniqueProcessId in pids]
        else:
            the_tasks = [t for t in tasks]

        # In case no PIDs found
        if len(the_tasks) < 1:
            return

        # Iterate through all the window objects matching for supported controls
        mh = messagehooks.MessageHooks(self._config)
        for winsta, atom_tables in mh.calculate():
            for desktop in winsta.desktops():
                for wnd, _level in desktop.windows(desktop.DeskInfo.spwnd):
                    if wnd.Process in the_tasks:

                        atom_class = mh.translate_atom(winsta, atom_tables, wnd.ClassAtom)
                        if atom_class:
                            atom_class = str(atom_class)
                            if '!' in atom_class:
                                comctl_class = atom_class.split('!')[-1].lower()
                                if comctl_class in supported_controls:
                                    
                                    # Do we need to fake being 32bit for Wow?
                                    if wnd.Process.IsWow64 and not self.fake_32bit:
                                        meta = addr_space.profile.metadata
                                        meta['memory_model'] = '32bit'
                                        self.apply_types(addr_space, meta)
                                        self.fake_32bit = True
                                    elif not wnd.Process.IsWow64 and self.fake_32bit:
                                            self.apply_types(addr_space)
                                            self.fake_32bit = False
                                            
                                    context = '{0}\\{1}\\{2}'.format(winsta.dwSessionId, winsta.Name, desktop.Name)
                                    task_vm = wnd.Process.get_process_address_space()
                                    wndextra_offset = wnd.v() + addr_space.profile.get_obj_size('tagWND')
                                    wndextra = obj.Object('address', offset=wndextra_offset, vm=task_vm)
                                    ctrl = obj.Object(supported_controls[comctl_class], offset=wndextra, vm=task_vm)
                                    if self._config.DUMP_DIR:
                                        dump_to_file(ctrl, wnd.Process.UniqueProcessId,
                                                     wnd.Process.ImageFileName, self._config.DUMP_DIR)
                                    yield context, atom_class, wnd.Process.UniqueProcessId, \
                                        wnd.Process.ImageFileName, wnd.Process.IsWow64, ctrl

    def render_table(self, outfd, data):
        """Output the results as a table
        
        @param  outfd: <file>
        @param  data: <generator>
        """

        self.table_header(outfd, [
            ('PID', '6'),
            ('Process', '14'),
            ('Control', ""),
        ])

        for context, atom_class, pid, proc_name, is_wow64, ctrl in data:
            # context, atom_class and is_wow64 are ignored
            self.table_row(outfd, pid, proc_name, str(ctrl))

    def unified_output(self, data):
        #output as volatility json format
        return TreeGrid([("Wnd Context", str),
                         ("Process ID", int),
                         ("ImageFileName", str),
                         ("IsWow64", str),
                         ("atom_class", str),
                         ("value-of WndExtra", str),
                         ("nChars", int),
                         ("selStart", int),
                         ("selEnd", int),
                         ("isPwdControl", int),
                         ("undoPos", int),
                         ("undoLen", int),
                         ("address-of undoBuf", str),
                         ("undoBuf", str),
                         ("Data", str),
                        ], self.generator(data))

    def generator(self, data):
        for context, atom_class, pid, proc_name, is_wow64, ctrl in data:
            yield (0, [
                str(context),
                int(pid),
                str(proc_name),
                str('Yes' if is_wow64 else 'No'),
                str(atom_class),
                str(hex(int(ctrl.v()))),
                int(ctrl.nChars),
                int(ctrl.selStart),
                int(ctrl.is_pwd()),
                int(ctrl.undoPos),
                int(ctrl.undoLen),
                int(ctrl.selEnd),
                str(ctrl.undoBuf),
                str(ctrl.get_undo(no_crlf=True)),
                str(ctrl.get_text()),
                ])

    def render_text(self, outfd, data):
        """Output the results as a text report
        
        @param  outfd: <file>
        @param  data: <generator>
        """

        for context, atom_class, pid, proc_name, is_wow64, ctrl in data:
            outfd.write('{}\n'.format('*' * 30))
            outfd.write('Wnd Context       : {}\n'.format(context))
            outfd.write('Process ID        : {}\n'.format(pid))
            outfd.write('ImageFileName     : {}\n'.format(proc_name))
            outfd.write('IsWow64           : {}\n'.format('Yes' if is_wow64 else 'No'))
            outfd.write('atom_class        : {}\n'.format(atom_class))
            outfd.write('value-of WndExtra : {:#x}\n'.format(ctrl.v()))
            ctrl.dump_meta(outfd)
            outfd.write('{}\n'.format('-' * 25))
            ctrl.dump_data(outfd)



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""
import volatility.obj as obj
import volatility.plugins.mac.common as common

class mac_timers(common.AbstractMacCommand):
    """ Reports timers set by kernel drivers """

    def calculate(self):
        common.set_plugin_members(self)

        kaddr_info = common.get_handler_name_addrs(self)

        real_ncpus = obj.Object("int", offset = self.addr_space.profile.get_symbol("_real_ncpus"), vm = self.addr_space)
        
        ptr = self.addr_space.profile.get_symbol("_cpu_data_ptr")
        cpu_data_ptrs = obj.Object(theType = 'Array', offset = ptr, vm = self.addr_space, targetType = "unsigned long long", count = real_ncpus)
        
        for i in range(real_ncpus):
            cpu_data = obj.Object('cpu_data', offset = cpu_data_ptrs[i], vm = self.addr_space)

            c = cpu_data.rtclock_timer
            q = c.queue
            
            ent = q.head.next
            first = ent
            seen = {}

            while ent.is_valid():
                seen[ent.v()] = 1

                timer = obj.Object("call_entry", offset = ent.v(), vm = self.addr_space)
                  
                func = timer.func.v()

                if func < 0x1000 or func == 0xffffffff00000000:
                    break

                (module, handler_sym) = common.get_handler_name(kaddr_info, func)
                
                if hasattr(timer, "entry_time"):
                    entry_time = timer.entry_time.v()
                else:
                    entry_time = -1
                
                yield func, timer.param0, timer.param1, timer.deadline, entry_time, module, handler_sym       
         
                ent = timer.q_link.next

                if ent == first or ent.v() in seen:
                    break


    def render_text(self, outfd, data):
        self.table_header(outfd, [("Function", "[addrpad]"), 
                                  ("Param 0", "[addrpad]"), 
                                  ("Param 1", "[addrpad]"),
                                  ("Deadline", "16"),
                                  ("Entry Time", "16"),
                                  ("Module", "16"),
                                  ("Symbol", ""),
                                 ])

        for func, p0, p1, deadline, entry_time, module, sym in data:
            self.table_row(outfd, func, p0, p1, deadline, entry_time, module, sym)














import os
import json

'''
Author: Gleeda <jamie@memoryanalysis.net>

modified from:
  http://stackoverflow.com/questions/25226208/represent-directory-tree-as-json

Quick and Dirty.  Run from the Volatility root directory and redirect:

python createtree.py > OUTPUT/d3/vol.json

'''

link = "https://github.com/volatilityfoundation/volatility/blob/master/"
ignore = [".git", "doxygen", ".gitignore", ".gitattributes"]

def path_to_dict(path):
    if path == ".":
        d = {'name': os.path.basename("root")}
    else:
        d = {'name': os.path.basename(path)}
        d['link'] = str(link + path).replace("/.", "")
    if os.path.isdir(path):
        d['type'] = "directory"
        d['children'] = [path_to_dict(os.path.join(path, x)) for x in os.listdir(path) if x not in ignore]
    else:
        d['type'] = "file"
    return d

print json.dumps(path_to_dict('.'))



# Volatility
# Copyright (C) 2007-2013 Volatility Foundation
#
# This file is part of Volatility.
#
# Volatility is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Volatility is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Volatility.  If not, see <http://www.gnu.org/licenses/>.
#

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0
@contact:      atcuno@gmail.com
@organization: 
"""
import volatility.obj   as obj
import volatility.utils as utils
import volatility.debug as debug

import volatility.plugins.mac.common as common

kaddr_info = None

class mac_interest_handlers(common.AbstractMacCommand):
    """ Lists IOKit Interest Handlers """

    def _struct_or_class(self, type_name):
        """Return the name of a structure or class. 

        More recent versions of OSX define some types as 
        classes instead of structures, so the naming is
        a little different.   
        """
        if self.addr_space.profile.vtypes.has_key(type_name):
            return type_name
        else:
            return type_name + "_class"
   
    def parse_properties(self, fdict):
        props = {}

        ents = obj.Object('Array', offset = fdict.dictionary, 
                          vm = self.addr_space, 
                          targetType = self._struct_or_class("dictEntry"), 
                          count = fdict.count)

        # walk the current set of notifications
        for ent in ents:
            if ent == None or not ent.is_valid():
                continue

            key = str(ent.key.dereference_as(self._struct_or_class("OSString")))
            val = ent.value
 
            props[key] = val
    
        return props

    def walk_reg_entry(self, reg_addr):
        regroot = obj.Object(self._struct_or_class("IORegistryEntry"), offset = reg_addr, vm = self.addr_space)
        
        fdict = regroot.fRegistryTable

        props = self.parse_properties(regroot.fPropertyTable)

        ents = obj.Object('Array', offset = fdict.dictionary, 
                          vm = self.addr_space, 
                          targetType = self._struct_or_class("dictEntry"), 
                          count = fdict.count)

        keys     = []
        children = []
        current_name = ""
        device_mem = False

        for ent in ents:
            if ent == None or not ent.is_valid():
                continue
            
            key = str(ent.key.dereference_as(self._struct_or_class("OSString")))
            
            keys.append(key)
          
            if key == "IODeviceMemory":
                current_name = str(ent.value.dereference_as(self._struct_or_class("OSString")))
                device_mem = True
 
            if key == "IOName" and device_mem == False:
                current_name = str(ent.value.dereference_as(self._struct_or_class("OSString")))

            if key == "IOServiceChildLinks":
                children.append(ent.value)

        if current_name == "":
            if "IOClass" in props:
                addr = props["IOClass"]
                s = obj.Object(self._struct_or_class("OSString"), offset = addr, vm = self.addr_space)
                current_name = "IOCLass: %s" % str(s)

        if current_name == "":
            serv = obj.Object(self._struct_or_class("IOService"), offset = reg_addr, vm = self.addr_space)
            buf  = self.addr_space.read(serv.pwrMgt.Name, 128)           
            if buf:
                idx = buf.find("\x00")
                if idx != -1:
                    buf = buf[:idx]

                current_name = buf

        prop_string = "".join(["%s=%x, " % (k,v) for (k,v) in props.items()])

        #print "%-20s | %s | %s" % (current_name, keys, prop_string)

        offset = self.addr_space.profile.get_obj_offset(self._struct_or_class("_IOServiceInterestNotifier"), "chain")

        for (k, v) in props.items():
            if k.find("nterest") != -1:
                cmd = obj.Object(self._struct_or_class("IOCommand"), offset = v, vm = self.addr_space)
                notifier_ptr = cmd.fCommandChain.next
                first_ptr = notifier_ptr

                last = 0

                while notifier_ptr.is_valid() and notifier_ptr != last:
                    notifier = obj.Object(self._struct_or_class("_IOServiceInterestNotifier"), offset = notifier_ptr - offset, vm = self.addr_space)

                    if not notifier.handler.is_valid():
                        break
   
                    last = notifier_ptr
                    notifier_ptr = notifier.chain.next

                    if notifier_ptr == first_ptr:
                        break

                    handler = notifier.handler.v()

                    (module, handler_sym) = common.get_handler_name(kaddr_info, handler)

                    yield k, handler, module, handler_sym          
 
        for child in children: 
            for k, handler, module, handler_sym in self.walk_child_links(child):
                yield k, handler, module, handler_sym

    def walk_child_links(self, addr):
        val = obj.Object(self._struct_or_class("OSArray"), offset = addr, vm = self.addr_space)

        arr_ptr = val.array
        cnt = val.count

        arr = obj.Object(theType = "Array", targetType = "Pointer", offset = arr_ptr, count = cnt, vm = self.addr_space)

        for a in arr:
            for key, handler, module, handler_sym in self.walk_reg_entry(a):
                yield key, handler, module, handler_sym
            
    def calculate(self):
        common.set_plugin_members(self)
        
        global kaddr_info
        kaddr_info = common.get_handler_name_addrs(self)

        regroot_addr = common.get_cpp_sym("gRegistryRoot", self.addr_space.profile)
        p = obj.Object("Pointer", offset = regroot_addr, vm = self.addr_space)

        for key, handler, module, handler_sym in self.walk_reg_entry(p):
            yield key, handler, module, handler_sym 

    def render_text(self, outfd, data):
        self.table_header(outfd, [("Interest", "24"),
                          ("Handler", "[addrpad]"),
                          ("Module", "32"),
                          ("Symbol", "")])

        for key, handler, module, handler_sym in data:
            self.table_row(outfd, key, handler, module, handler_sym)

